# -*- coding: utf-8 -*-
#COPYRIGHT 2016 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
from arcpy.arcobjects._base import _ObjectWithoutInitCall, passthrough_attr, pass_parameterized_attr
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
from arcpy.geoprocessing._base import gp_fixargs


class Chart(_ObjectWithoutInitCall):
    """The Chart class defines an ArcGIS Pro chart. The class allows you to create different types of charts, including
    bar charts, line charts, histograms, and scatter plots. Also, you can use the class to define the chart title, axes,
    and other properties.
    """
    _hId = passthrough_attr('horizontalAxisId')
    _vId = passthrough_attr('verticalAxisId')
    _chartType = passthrough_attr('chartType')
    title = passthrough_attr('title')
    description = passthrough_attr('description')

    class _arc_base(object):
        def __init__(self, parent):
            self._parent = parent
            self._arc_object = parent._arc_object

    class _seriesType(_arc_base):
        pass

        def __init__(self, parent):
            super().__init__(parent)

        def __repr__(self):
            return 'Chart("{} series")'.format(self._parent._chartType)

        def __str__(self):
            return 'Chart("{} series")'.format(self._parent._chartType)

    class _axis(_arc_base):
        """Sets additional properties that apply to axis only.
        field
            The name of the attribute field to display on the axis.

        sort
            The sorting method applied to this axis. ASC for ascending sorting and DESC for descending sorting. This
            only applies to bar charts.

        title
            The label of the axis that displays on the chart.
        """
        _axisType = ''
        title = pass_parameterized_attr('axisProperty', axisId="axisId", type='title')
        field = pass_parameterized_attr('axisProperty', axisId="axisId", type='field')
        sort = pass_parameterized_attr('axisProperty', axisId="axisId", type='sort')

        def __repr__(self):
            return "Chart.{}axis".format(self._axisType)

        def __str__(self):
            return "Chart<{}>.{}axis<{}>".format(self._parent._chartType, self._axisType, self.title)

        def __init__(self, parent, i):
            super().__init__(parent)
            if i == 0:
                self._axisType = 'X'
            else:
                self._axisType = 'Y'
            self.axisId = i

    class _bar(_seriesType):
        """Sets additional properties that apply to bar charts only.
        aggregation
            The statistical calculation applied to values that occur at the same value along the x-axis.
            Valid statistics include COUNT, SUM, MEAN, and MEDIAN. No aggregation means that each value will be plotted
            on the chart regardless of overlap or recurring values.

        splitCategory
            A second categorical attribute field that adds a separate series or bar for each unique value in the field.
        """
        aggregation = passthrough_attr('aggregation')
        splitCategory = passthrough_attr('whereClause')
        def __init__(self, parent):
            super().__init__(parent)

    class _scatterPlot(_seriesType):
        """Sets additional properties that apply to scatter charts only."""
        def __init__(self, parent):
            super().__init__(parent)

    class _line(_seriesType):
        """Sets additional properties that apply to histograms only.
        splitCategory
            A second categorical attribute field that adds a separate series or bar for each unique value in the field.

        timeAggregationType
             Determines the time interval alignment. Valid options include equalIntervalsFromStartTime and
             equalIntervalsFromEndTime.

        timeIntervalSize
             A number for the span of time that will be binned or aggregated together. Must be used in conjunction with
             timeIntervalUnit.

        timeIntervalUnits
             The time unit that corresponds to the timeIntervalSize. Valid options include SECONDS, MINUTES, HOURS,
             DAYS, WEEKS, MONTHS, YEARS.

        trimInCompleteTimeInterval
             Determines if incomplete time intervals at the start or end of the data span (depending on the
             timeAggregationType) will be removed from the chart. Incomplete intervals on a chart can cause misleading
             results in which the period in question is under or over-reported due to a different amount of time in the
             interval. True indicates to trim these incomplete intervals, while False does not trim incomplete
             intervals.
        """
        aggregation = passthrough_attr('aggregation')
        splitCategory = passthrough_attr('whereClause')
        timeAggregationType = passthrough_attr('timeAggregationType')
        timeIntervalUnits = passthrough_attr('timeIntervalUnits')
        timeIntervalSize = passthrough_attr('timeIntervalSize')
        trimInCompleteTimeInterval = passthrough_attr('trimInCompleteTimeInterval')

        def __init__(self, parent):
            super().__init__(parent)

    class _histogram(_seriesType):
        """Sets additional properties that apply to histograms only.
        binCount
            Number that determines the number of bins to display in the histogram.

        showMean
            Toggles if the mean statistical line is visible in the histogram. True displays the line, False hides the
            line.

        showMedian
            Toggles if the median statistical line is visible in the histogram. True displays the line, False hides the
            line.

        showStandardDeviation
            Toggles if the standard deviation statistical line is visible in the histogram. True displays the line,
            False hides the line.
        """
        binCount = passthrough_attr('binCount')
        showMean = passthrough_attr('showMean')
        showMedian = passthrough_attr('showMedian')
        showStandardDeviation = passthrough_attr('showStandardDeviation')

        def __init__(self, parent):
            super().__init__(parent)

    class _legend(_arc_base):
        """Sets properties of the chart legend.
        visible
            Toggles if the legend is visible in the chart view. True displays the legend, False hides the legend.
        """
        visible = passthrough_attr('legendVisible')

        def __init__(self, parent):
            super().__init__(parent)

    @property
    def type(self):
        return self._chartType

    @type.setter
    def type(self, val):
        """Chart.type{String}
        Sets the type of chart to create. Valid options include.
            bar
            line
            scatter
            histogram
        """
        if getattr(self, '_chartType'):
            raise AttributeError('Cannot change chart type: "{}"'.format(self._chartType))
        else:
            self._createChartType(val)

    @property
    def legend(self):
        return self._create_legend()

    def _create_legend(self):
        if hasattr(self, 'chartLegend'):
            return self.chartLegend
        else:
            setattr(self, 'chartLegend', self._legend(self))
            return self.chartLegend

    def _createChartType(self, val):
        if val == 'bar':
            self._create_bar()
        elif val == 'scatter':
            self._create_scatterPlot()
        elif val == 'line':
            self._create_line()
        elif val == 'histogram':
            self._create_histogram()

    def _create_axis(self, i):
        return self._axis(self, i)

    def _create_bar(self):
        setattr(self, '_chartType', 'bar')
        setattr(self, 'bar', self._bar(self))
        return self

    def _create_scatterPlot(self):
        setattr(self, '_chartType', 'scatter')
        setattr(self, 'scatter', self._scatterPlot(self))
        return self

    def _create_line(self):
        setattr(self, '_chartType', 'line')
        setattr(self, 'line', self._line(self))
        return self

    def _create_histogram(self):
        setattr(self, '_chartType', 'histogram')
        setattr(self, 'histogram', self._histogram(self))
        return self

    def addToLayer(self, layer_or_layerfile):
        """Chart.addToLayer(layer_or_layerfile)
        Adds the chart class to a layer.

        layer_or_layerfile{object}
            The chart will be added to this target layer object.

        The chart object properties will be set into the layer definition.
        """
        return convertArcObjectToPythonObject(self._arc_object.addToLayer(*gp_fixargs((layer_or_layerfile,), True)))

    def updateChart(self):
        """Chart.updateChart()
        Updates chart properties to sync changes between the class and the chart associated with a layer.

        After defining chart properties in the chart class, the final step is typically to add the chart to a layer.
        Use the addToLayer method to accomplish this. However, you may wish to further modify the chart properties.
        Instead of starting from scratch with a new chart, you can modify the properties of the original chart class,
        then use the updateChart method to sync any changes into the layer. This will allow the changes you make to be
        presented in the Chart properties pane and chart view.Use updateChart to sync chart property changes into a
        layer.
        """
        return convertArcObjectToPythonObject(self._arc_object.updateChart(*gp_fixargs((), True)))

    def __init__(self, name):
        import arcgisscripting
        self._arc_object = arcgisscripting._chart.Chart(*gp_fixargs((name,), True))
        self.xAxis = self._create_axis(self._hId)
        self.yAxis = self._create_axis(self._vId)

ob_conversion = (
    ('ChartObject', Chart),
)
