# -*- coding: utf-8 -*-
#COPYRIGHT 2017 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
r"""The ArcGIS Pro Aviation toolbox contains tools that generate
obstruction identification surfaces (OIS) according to different
specifications."""
__all__ = ['FAA18B', 'FAAFAR77', 'ICAOAnnex14', 'ICAOAnnex15']
__alias__ = 'aviation'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs, gptoolnotsupported
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Airports\Obstruction Identification Surfaces toolset
@gptoolnotsupported
@gptooldoc('FAA18B_aviation', None)
def FAA18B(in_features=None, target=None, runway_type=None, highend_clear_way_length=None, lowend_clear_way_length=None, airport_elevation=None, include_merged_surface=None):
    """FAA18B_aviation(in_features, target, runway_type, {highend_clear_way_length}, {lowend_clear_way_length}, {airport_elevation}, {include_merged_surface})

        Creates obstruction identification surfaces based on the FAA 18B
        specification. These surfaces describe airspaces that should be free
        of obstructions. Surfaces are used to support planning and design
        activities. The type, function, and dimension of a surface differ by
        runway classification.

     INPUTS:
      in_features (Feature Layer):
          The input runway dataset. The feature class must be z-enabled and
          contain polylines.
      target (Feature Layer):
          The output feature class that will contain the generated obstacle
          identification surfaces.
      runway_type (String):
          The runway classification of in_features.

          * NON_VERTICAL_GUIDANCE_TYPE_1-A runway designed for visual maneuvers,
          nonvertically guided operations, and instrument departure procedures.

          * NON_VERTICAL_GUIDANCE_TYPE_2-A specially prepared hard surface
          (SPHS) runway designed for visual maneuvers, nonvertically guided
          operations, and instrument departure procedures. SPHS runways have a
          primary surface that extends 200 feet beyond each end of the runway.

          * VERTICAL_GUIDANCE-A runway that uses precision guidance systems to
          support aircraft approach and landing.
      highend_clear_way_length {Double}:
          The length, in feet, of the area at the high end of the runway. An
          aircraft can make a portion of its initial climb over this area.
      lowend_clear_way_length {Double}:
          The length, in feet, of the area at the low end of the runway. An
          aircraft can make a portion of its initial climb over this area.
      airport_elevation {Double}:
          The highest point, in feet, on any runway in an airport. The units for
          elevation are based on the specification. The tool will automatically
          populate this value from a z-value in your aeronautical database if
          you are using the AIS or Airports data model. Airport elevation is
          stored in the z-value of the ADHP point feature class in the AIS data
          model. In the Airports data model, elevation is stored in the z-value
          of the Airport Control Point feature class for records with the
          Point_Type field populated with AIRPORT_ELEVATION.
      include_merged_surface {Boolean}:
          Indicates whether merged horizontal and conical surfaces are included
          in the OIS in addition to the regular surfaces.

          * INCLUDE_MERGED_SURFACE-Include merged surfaces in the OIS output.
          This is the default.

          * NOT_INCLUDE_MERGED_SURFACE-Do not include merged surfaces in the
          OIS output."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.FAA18B_aviation(*gp_fixargs((in_features, target, runway_type, highend_clear_way_length, lowend_clear_way_length, airport_elevation, include_merged_surface), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('FAAFAR77_aviation', None)
def FAAFAR77(in_features=None, target=None, high_runway_end_type=None, low_runway_end_type=None, specially_prepared_hard_surface_runway=None, highend_clear_way_length=None, lowend_clear_way_length=None, airport_elevation=None, include_merged_surface=None):
    """FAAFAR77_aviation(in_features, target, high_runway_end_type, low_runway_end_type, {specially_prepared_hard_surface_runway}, {highend_clear_way_length}, {lowend_clear_way_length}, {airport_elevation}, {include_merged_surface})

        Creates obstruction identification surfaces based on the FAA FAR 77
        specification. These surfaces describe airspace segments that are
        clear of obstructions. The type, function, and dimension of a surface
        differs by runway classification. This tool creates surfaces as
        polygon or multipatch features.

     INPUTS:
      in_features (Feature Layer):
          The input runway dataset. The feature class must be z-enabled and
          contain polylines.
      target (Feature Layer):
          The output feature class that will contain the generated obstruction
          identification surfaces.
      high_runway_end_type (String):
          The classification of the high end of the runway.

          * CONSTRUCTION_OR_ALTERATION_ON_AN_AIRPORT_WITH_LONGEST_RUNWAY_MORE_TH
          AN_3200_FEET-Construction on or alteration to a runway longer than
          3,200 feet with an imaginary surface that extends outward 20,000 feet
          and has a slope that does not exceed 100 to 1.

          * CONSTRUCTION_OR_ALTERATION_ON_AN_AIRPORT_WITH_LONGEST_RUNWAY_LESS_TH
          AN_3200_FEET-Construction on or alteration to a runway less than 3,200
          feet long with an imaginary surface that extends outward 10,000 feet
          and has a slope that does not exceed 50 to 1.

          * CONSTRUCTION_OR_ALTERATION_ON_A_HELIPORT-Construction on or
          alteration to a heliport landing and takeoff area with an imaginary
          surface that extends outward 5,000 feet and has a slope that does not
          exceed 25 to 1.

          * MILITARY_AIRPORT-Military airport runways are operated by an armed
          force of the United States. Primary surfaces are the same length as
          the runway. Primary surface width is 2,000 feet. Clear zone surface
          length is 1,000 feet, and width is the same as the primary surface.
          The approach clearance surface starts 200 feet beyond each end of the
          primary surface and extends for 50,000 feet. Approach surface width
          matches the primary surface width at the runway end but flares to a
          width of 16,000 feet at an elevation of 50,000 feet. Approach
          clearance surface slope is 50 to I to an elevation of 500 feet above
          airport elevation. It then rises horizontally to 50,000 feet.
          Transitional surface slope is 7 to I outward and upward at right
          angles to the runway centerline. See section 77.28 in the FAR Part 77
          specification for more information.

          *
          NON_PRECISION_INSTRUMENT_RUNWAY_GREATER_THAN_(>)_3/4_MILE_VISIBILITY-A
          runway with a nonprecision instrument approach procedure that allows
          for landing in visibility conditions greater than three-quarters of a
          mile.

          *  NON_PRECISION_INSTRUMENT_RUNWAY_LESS_THAN_(<)
          _3/4_MILE_VISIBILITY-A runway with a nonprecision instrument approach
          procedure that allows for landing in visibility conditions less than
          three-quarters of a mile.

          * PRECISION_INSTRUMENT_RUNWAY-A runway that uses Instrument Landing
          System (ILS) or Precision Approach Radar (PAR) for approach
          procedures.

          * UTILITY_RUNWAY_VISUAL_APPROACH-A runway built for propeller aircraft
          not exceeding 12,500 pounds gross weight. Aircraft using the runway
          employ visual approach procedures.

          * UTILITY_RUNWAY_NON_PRECISION_INSTRUMENT_APPROACH-A runway built for
          propeller aircraft not exceeding 12,500 pounds gross weight. The
          runway has an instrument approach procedure that uses air navigation
          facilities with horizontal guidance. It can also have area-type
          navigation equipment with approved nonprecision instrument approach
          procedures.

          * VISUAL_RUNWAY_VISUAL_APPROACH-A runway that supports only visual
          approach procedures.
      low_runway_end_type (String):
          The classification of the low end of the runway.

          * SAME_AS_HIGH_RUNWAY_END_CLASSIFICATION-No low runway end type.

          *
          NON_PRECISION_INSTRUMENT_RUNWAY_GREATER_THAN_(>)_3/4_MILE_VISIBILITY-A
          runway with a nonprecision instrument approach procedure that allows
          for landing in visibility conditions greater than three-quarters of a
          mile.

          * NON_PRECISION_INSTRUMENT_RUNWAY_LESS_THAN_(<)_3/4_MILE_VISIBILITY-A
          runway with a nonprecision instrument approach procedure that allows
          for landing in visibility conditions less than three-quarters of a
          mile.

          * PRECISION_INSTRUMENT_RUNWAY-A runway that uses Instrument Landing
          System (ILS) or Precision Approach Radar (PAR) for approach
          procedures.

          * UTILITY_RUNWAY_VISUAL_APPROACH-A runway built for propeller aircraft
          not exceeding 12,500 pounds gross weight. Aircraft using the runway
          employ visual approach procedures.

          * UTILITY_RUNWAY_NON_PRECISION_INSTRUMENT_APPROACH-A runway built for
          propeller aircraft not exceeding 12,500 pounds gross weight. The
          runway has an instrument approach procedure that uses air navigation
          facilities with horizontal guidance. It can also have area-type
          navigation equipment with approved nonprecision instrument approach
          procedures.

          * VISUAL_RUNWAY_VISUAL_APPROACH-A runway that supports only visual
          approach procedures.
      specially_prepared_hard_surface_runway {Boolean}:
          A specially prepared hard surface indicates that the primary surface
          extends 200 feet beyond each end of the runway.

          *  SPECIALLY_PREPARED_HARD_SURFACE_RUNWAY-The runway has a specially
          prepared hard surface. This is the default.

          *  NON_SPECIALLY_PREPARED_HARD_SURFACE_RUNWAY-The runway does not have
          a specially prepared hard surface.
      highend_clear_way_length {Double}:
          The length, in feet, of the area at the high end of the runway. An
          aircraft can make a portion of its initial climb over this area.
      lowend_clear_way_length {Double}:
          The length, in feet, of the area at the low end of the runway. An
          aircraft can make a portion of its initial climb over this area.
      airport_elevation {Double}:
          The highest point, in feet, on any runway in an airport. The units for
          elevation are based on the specification. The tool will automatically
          populate this value from a z-value in your aeronautical database if
          you are using the AIS or Airports data model. Airport elevation is
          stored in the z-value of the ADHP point feature class in the AIS data
          model. In the Airports data model, elevation is stored in the z-value
          of the Airport Control Point feature class for records with the
          Point_Type field populated with AIRPORT_ELEVATION.
      include_merged_surface {Boolean}:
          Indicates whether merged horizontal and conical surfaces are included
          in the OIS in addition to the regular surfaces.

          * INCLUDE_MERGED_SURFACE-Include merged surfaces in the OIS output.
          This is the default.

          * NOT_INCLUDE_MERGED_SURFACE-Do not include merged surfaces in the OIS
          output."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.FAAFAR77_aviation(*gp_fixargs((in_features, target, high_runway_end_type, low_runway_end_type, specially_prepared_hard_surface_runway, highend_clear_way_length, lowend_clear_way_length, airport_elevation, include_merged_surface), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('ICAOAnnex14_aviation', None)
def ICAOAnnex14(in_features=None, target=None, runway_type=None, highend_clear_way_length=None, lowend_clear_way_length=None, airport_elevation=None, runway_direction=None, include_merged_surface=None):
    """ICAOAnnex14_aviation(in_features, target, runway_type, {highend_clear_way_length}, {lowend_clear_way_length}, {airport_elevation}, {runway_direction}, {include_merged_surface})

        Creates obstruction identification surfaces based on the ICAO Annex
        14 specification. These surfaces define areas of airspace around an
        aerodrome. Each area defines the limits to which an obstruction can
        project into an airspace. The type, function, and dimension of a
        surface differ by runway classification. This tool creates surfaces as
        polygon or multipatch features.

     INPUTS:
      in_features (Feature Layer):
          The input runway dataset. The feature class must be Z enabled and
          contain points, polylines, or polygons.
      target (Feature Layer):
          The output feature class that will contain the generated obstruction
          identification surfaces.
      runway_type (String):
          The runway classification of the in_features.

          * NON_INSTRUMENT_CODE_NUMBER_1-A runway intended for the operation of
          aircraft using visual approach procedures. Runway strip length is 30
          meters.

          * NON_INSTRUMENT_CODE_NUMBER_2-A runway with a 60-meter strip length
          and 40-meter strip width that is intended for the operation of
          aircraft using visual approach procedures.

          * NON_INSTRUMENT_CODE_NUMBER_3-A runway with a 60-meter strip length
          and 75-meter strip width that is intended for the operation of
          aircraft using visual approach procedures.

          * NON_INSTRUMENT_CODE_NUMBER_4-A runway with a 60-meter strip length
          and 75-meter strip width that is intended for the operation of
          aircraft using visual approach procedures.

          * NON_PRECISION_APPROACH_CODE_NUMBER_1-An instrument runway served by
          visual aids and a nonvisual aid providing at least directional
          guidance adequate for a straight-in approach. This runway type has a
          60-meter strip length and a 75-meter strip width on either side of the
          runway centre line.

          * NON_PRECISION_APPROACH_CODE_NUMBER_2-An instrument runway served by
          visual aids and a nonvisual aid providing at least directional
          guidance adequate for a straight-in approach. This runway type has a
          60-meter strip length and a 75-meter strip width on either side of the
          runway centre line.

          * NON_PRECISION_APPROACH_CODE_NUMBER_3-An instrument runway served by
          visual aids and a nonvisual aid providing at least directional
          guidance adequate for a straight-in approach. This runway type has a
          60-meter strip length and a 150-meter strip width on either side of
          the runway centre line.

          * NON_PRECISION_APPROACH_CODE_NUMBER_4-An instrument runway served by
          visual aids and a nonvisual aid providing at least directional
          guidance adequate for a straight-in approach. This runway type has a
          60-meter strip length and a 150-meter strip width on either side of
          the runway centre line.

          * PRECISION_APPROACH_CATEGORY_I_CODE_NUMBER_1-An instrument runway
          served by an Instrument Landing System (ILS) or a Microwave Landing
          System (MLS) and visual aids intended for operations with a decision
          height not lower than 60 meters (200 ft.) and either a visibility not
          less than 800 meters or a runway visual range not less than 550
          meters. This runway type has a 60-meter strip length and a 75-meter
          strip width on either side of the runway centre line.

          * PRECISION_APPROACH_CATEGORY_I_CODE_NUMBER_2-An instrument runway
          served by ILS and/or MLS and visual aids intended for operations with
          a decision height not lower than 60 meters (200 ft.) and either a
          visibility not less than 800 meters or a runway visual range not less
          than 550 meters. This runway type has a 60-meter strip length and a
          75-meter strip width on either side of the runway centre line.

          * PRECISION_APPROACH_CATEGORY_I_CODE_NUMBER_3_4-An instrument runway
          served by ILS and/or MLS and visual aids intended for operations with
          a decision height not lower than 60 meters (200 ft.) and either a
          visibility not less than 800 meters or a runway visual range not less
          than 550 meters. This runway type has a 60-meter strip length and a
          150-meter strip width on either side of the runway centre line.

          * PRECISION_APPROACH_CATEGORY_II_III_CODE NUMBER_3_4-An instrument
          runway served by ILS and/or MLS and visual aids intended for
          operations with a decision height lower than 60 meters (200 ft.) but
          not lower than 30 meters (100 ft.) and a runway visual range not less
          than 350 meters. This runway type has a 60-meter strip length and a
          150-meter strip width on either side of the runway centre line.
      highend_clear_way_length {Double}:
          The length, in meters, of the area at the high end of the runway. An
          aircraft can make a portion of its initial climb over this area.
      lowend_clear_way_length {Double}:
          The length, in meters, of the area at the low end of the runway. An
          aircraft can make a portion of its initial climb over this area.
      airport_elevation {Double}:
          The highest point, in meters, on any runway in an airport. The units
          for elevation are based on the specification. The tool will
          automatically populate this value from a z-value in your aeronautical
          database if you are using the AIS or Airports data model. Airport
          elevation is stored in the z-value of the ADHP point feature class in
          the AIS data model. In the Airports data model, elevation is stored in
          the z-value of the Airport Control Point feature class for records
          with the Point_Type field populated with AIRPORT_ELEVATION.
      runway_direction {String}:
          The direction of the in_features.

          * HIGH_END_TO_LOW_END-From runway high end to low end

          * LOW_END_TO_HIGH_END-From runway low end to high end
      include_merged_surface {Boolean}:
          Indicates whether merged surfaces are generated.

          * INCLUDE_MERGED_SURFACE-All the surfaces are generated for the merged
          surfaces, as well as merged conical and horizontal surfaces. This is
          the default.

          * NOT_INCLUDE_MERGED_SURFACE-Surfaces are not generated for the merged
          surfaces."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ICAOAnnex14_aviation(*gp_fixargs((in_features, target, runway_type, highend_clear_way_length, lowend_clear_way_length, airport_elevation, runway_direction, include_merged_surface), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('ICAOAnnex15_aviation', None)
def ICAOAnnex15(in_features=None, target=None, highend_clear_way_length=None, lowend_clear_way_length=None):
    """ICAOAnnex15_aviation(in_features, target, {highend_clear_way_length}, {lowend_clear_way_length})

        Creates obstruction identification surfaces based on the ICAO Annex
        15 specification. These surfaces define areas of airspace around an
        aerodrome. Each area defines the limits to which an obstruction can
        project into an airspace. This tool creates surfaces as polygon or
        multipatch features.

     INPUTS:
      in_features (Feature Layer):
          The input runway dataset. The feature class must be z-enabled and
          contain polylines.
      target (Feature Layer):
          The output feature class that will contain the generated obstruction
          identification surfaces.
      highend_clear_way_length {Double}:
          The length, in meters, of the area at the high end of the runway. An
          aircraft can make a portion of its initial climb over this area.
      lowend_clear_way_length {Double}:
          The length of the area at the low end of the runway. An aircraft can
          make a portion of its initial climb over this area."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ICAOAnnex15_aviation(*gp_fixargs((in_features, target, highend_clear_way_length, lowend_clear_way_length), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject, gptoolnotsupported
