# -*- coding: utf-8 -*-
#COPYRIGHT 2017 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
r"""The tools in the Cartography toolbox are designed to produce and
refine data to support the production of maps. This includes the
creation of masks, the simplification and aggregation of features and
reduction of their density, as well as tools for the creation of map
series."""
__all__ = ['AggregatePoints', 'AggregatePolygons', 'AlignMarkerToStrokeOrFill',
           'CalculateLineCaps', 'CalculatePolygonMainAngle',
           'CollapseRoadDetail', 'ConvertLabelsToAnnotation',
           'CreateCartographicPartitions', 'CulDeSacMasks',
           'DelineateBuiltUpAreas', 'DisperseMarkers', 'FeatureOutlineMasks',
           'GridIndexFeatures', 'IntersectingLayersMasks', 'MergeDividedRoads',
           'SetControlPointAtIntersect', 'SetControlPointByAngle',
           'SimplifyBuilding', 'SimplifyLine', 'SimplifyPolygon', 'SmoothLine',
           'SmoothPolygon', 'ThinRoadNetwork']
__alias__ = 'cartography'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs, gptoolnotsupported
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Annotation toolset
@gptooldoc('ConvertLabelsToAnnotation_cartography', None)
def ConvertLabelsToAnnotation(input_map=None, conversion_scale=None, output_geodatabase=None, anno_suffix=None, extent=None, generate_unplaced=None, require_symbol_id=None, feature_linked=None, auto_create=None, update_on_shape_change=None, output_group_layer=None):
    """ConvertLabelsToAnnotation_cartography(input_map, conversion_scale, output_geodatabase, {anno_suffix}, {extent}, {generate_unplaced}, {require_symbol_id}, {feature_linked}, {auto_create}, {update_on_shape_change}, {output_group_layer})

        Converts labels to annotation for the entire map. Both standard
        annotation and feature-linked annotation can be created.

     INPUTS:
      input_map (Map):
          The input map.
      conversion_scale (Double):
          The scale at which to convert labels. If a reference scale is set on
          the map, the reference scale will be used for symbol sizing and
          annotation feature class creation, but conversion will happen at this
          scale.
      output_geodatabase (LocalDatabase|RemoteDatabase):
          The workspace where the output feature classes will be saved. The
          workspace can be an existing geodatabase or an existing feature
          dataset. If this is not the same database as used by all the layers in
          the map, the feature-linked option will be disabled.
      anno_suffix {String}:
          The suffix that will be added to each new annotation feature class.
          This suffix will be appended to the name of the source feature class
          for each new annotation feature class.
      extent {Extent}:
          The extent that contains the labels to convert to annotation. It can
          be the extent of the map, union of input layers, intersection of input
          layers, current display extent, same as specified layer extent, or it
          can be specified by value.

          * MAXOF-Union of inputs

          * MINOF-Intersection of inputs

          * DISPLAY-Same extent as current display

          * <Layer>-Same extent as specified layer
      generate_unplaced {Boolean}:
          Choose whether to create unplaced annotation from unplaced labels.

          * ONLY_PLACED-Annotation will only be created for features that are
          currently labeled. This is the default.

          * GENERATE_UNPLACED-Unplaced annotation are stored in the annotation
          feature class. The status field for these annotation is set to
          Unplaced.
      require_symbol_id {Boolean}:
          Choose whether to place a restriction on what text symbol properties
          can be edited.

          * NO_REQUIRE_ID-Allow any text symbol property to be edited. This is
          the default.

          *  REQUIRE_ID-Only allow edits to symbol properties that enable
          annotation features to maintain reference to their associated text
          symbol in the collection.
      feature_linked {Boolean}:
          This parameter is only available with ArcGIS Desktop Standard and
          ArcGIS Desktop Advanced licenses. Choose whether the output annotation
          feature class will be linked to
          the features in another feature class.

          *  STANDARD-The output annotation feature class will not be linked to
          the features in another feature class. This is the default.

          *  FEATURE_LINKED-The output annotation feature class will be linked
          to the features in another feature class.
      auto_create {Boolean}:
          Choose whether to create annotation when new features are added to
          the linked feature class if the feature_linked parameter is set to
          FEATURE_LINKED.

          * AUTO_CREATE-Feature-linked annotation will be created when new
          features are added to the linked feature class. This is the default.

          *  NO_AUTO_CREATE-Feature-linked annotation will not be created when
          new features are added to the linked feature class.
      update_on_shape_change {Boolean}:
          Choose whether to update the position of annotation when the shape of
          the linked feature is updated if the feature_linked parameter is set
          to FEATURE_LINKED.

          * SHAPE_UPDATE-Update the position of the annotation when the shape of
          the linked feature is modified. This is the default.

          * NO_SHAPE_UPDATE-Annotation stays in its original position when the
          shape of the linked feature is modified.

     OUTPUTS:
      output_group_layer {Group Layer}:
          The group layer that will contain the generated annotation. You can
          use the Save To Layer File tool to write the output group layer to a
          layer file."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ConvertLabelsToAnnotation_cartography(*gp_fixargs((input_map, conversion_scale, output_geodatabase, anno_suffix, extent, generate_unplaced, require_symbol_id, feature_linked, auto_create, update_on_shape_change, output_group_layer), True)))
        return retval
    except Exception as e:
        raise e


# Cartographic Refinement toolset
@gptoolnotsupported
@gptooldoc('AlignMarkerToStrokeOrFill_cartography', None)
def AlignMarkerToStrokeOrFill(in_point_features=None, in_line_or_polygon_features=None, search_distance=None, marker_orientation=None):
    """AlignMarkerToStrokeOrFill_cartography(in_point_features, in_line_or_polygon_features, search_distance, {marker_orientation})

        Aligns the marker symbol layers of a point feature class to the
        nearest stroke or fill symbol layers in a line or polygon feature
        class within a specified search distance.

     INPUTS:
      in_point_features (Layer):
          The input point feature layer containing point symbols to be aligned
          to nearby lines or polygons.
      in_line_or_polygon_features (Layer):
          The input line or polygon feature layer that the input point symbols
          will be aligned to.
      search_distance (Linear Unit):
          The search distance from graphical marker edge to graphical stroke or
          fill edge. A distance greater than or equal to zero must be specified.
      marker_orientation {String}:
          Specifies how to orient the marker symbol layer relative to the stroke
          or fill symbol layer's edge.

          * PERPENDICULAR-aligns marker symbol layers perpendicularly to the
          stroke or fill edge. This is the default.

          * PARALLEL-aligns marker symbol layers parallel to the stroke or fill
          edge."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.AlignMarkerToStrokeOrFill_cartography(*gp_fixargs((in_point_features, in_line_or_polygon_features, search_distance, marker_orientation), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('CalculateLineCaps_cartography', None)
def CalculateLineCaps(in_features=None, cap_type=None, dangle_option=None):
    """CalculateLineCaps_cartography(in_features, {cap_type}, {dangle_option})

        Modifies the cap type for stroke symbol layers in the line symbols of
        the input layer.

     INPUTS:
      in_features (Layer):
          The input feature layer containing line symbols. Stroke symbol layers
          must have the Cap Type property connected to an attribute field. The
          values in this field are updated by this tool.
      cap_type {String}:
          Defines how the ends of stroke symbol layers are drawn. The default
          cap type of strokes is round, where the symbol is terminated with a
          semicircle of radius equal to stroke width is centered at the line
          endpoint. This tool changes cap type to Butt or Square.

          * BUTT-Specifies to end the stroke symbol exactly where the line
          geometry ends. This is the default.

          * SQUARE-Specifies to end the stroke symbol with closed, square caps
          that extend past the endpoint of the line by half of the symbol width.
      dangle_option {String}:
          Controls how line caps are calculated for adjoining line features that
          share an endpoint but are drawn with different symbology.

          * CASED_LINE_DANGLE-Modifies the cap style for dangling lines (those
          not connected at their endpoints to another line) and also for the
          lines where a cased-line symbol is joined at the endpoint of a single-
          stroke layer line symbol. This is the default.

          * TRUE_DANGLE-Modifies the cap style only for endpoints that are not
          connected to another feature."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CalculateLineCaps_cartography(*gp_fixargs((in_features, cap_type, dangle_option), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('CalculatePolygonMainAngle_cartography', None)
def CalculatePolygonMainAngle(in_features=None, angle_field=None, rotation_method=None):
    """CalculatePolygonMainAngle_cartography(in_features, angle_field, {rotation_method})

        Calculates the dominant angles of input polygon features and assigns
        the values to a specified field in the feature class.

     INPUTS:
      in_features (Feature Layer):
          The input polygon features.
      angle_field (Field):
          The field that will be updated with the polygon main angle values.
      rotation_method {String}:
          Controls the method and origin point of rotation.

          * GEOGRAPHIC-Angle is calculated clockwise with 0 at top/north.

          * ARITHMETIC-Angle is calculated counterclockwise with 0 at the
          right/east.

          * GRAPHIC-Angle is calculated counterclockwise with 0 at top/north.
          This is the default."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CalculatePolygonMainAngle_cartography(*gp_fixargs((in_features, angle_field, rotation_method), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('DisperseMarkers_cartography', None)
def DisperseMarkers(in_point_features=None, minimum_spacing=None, dispersal_pattern=None):
    """DisperseMarkers_cartography(in_point_features, minimum_spacing, {dispersal_pattern})

        Finds point symbols that are overlapping or too close to one another
        and spreads them apart based on a minimum spacing and dispersal
        pattern.

     INPUTS:
      in_point_features (Layer):
          The input point feature layer to be dispersed.
      minimum_spacing (Linear Unit):
          The minimum separation distance between individual point symbols, in
          page units. A distance must be specified and must be greater than or
          equal to zero. When a positive value is specified, markers will be
          separated by that value; when a value of zero is specified, point
          symbols will be touching. The default page unit is Points.
      dispersal_pattern {String}:
          Specifies the pattern in which the dispersed point symbols are placed.
          A group of point symbols will have a center of mass derived from the
          locations of each point in the group. The center of mass is then used
          as the anchor point around which the dispersal pattern operates.

          * EXPANDED-The general pattern of the point symbols will be maintained
          as they are spread apart. Points that were exactly coincident are
          dispersed to a circle around their center of mass. This is the
          default.

          * RANDOM-Point symbols are placed around the center of mass in a
          random dispersal that respects the minimum spacing.

          * SQUARES-Point symbols are placed in multiple square rings around the
          center of mass, ensuring that all points are placed as closely
          together as allowable by the minimum spacing parameter.

          * RINGS-Point symbols are placed in multiple circular rings around the
          center of mass, ensuring that all points are placed as closely
          together as allowable by the minimum spacing parameter.

          * SQUARE-Point symbols are placed evenly around the center of mass in
          a single square pattern.

          * RING-Point symbols are placed evenly around the center of mass in a
          single circular pattern.

          * CROSS-Point symbols are spaced evenly on horizontal and vertical
          axes originating from the center of mass.

          * X_CROSS-Point symbols are spaced evenly on 45° axes originating from
          the center of mass."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.DisperseMarkers_cartography(*gp_fixargs((in_point_features, minimum_spacing, dispersal_pattern), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('SetControlPointAtIntersect_cartography', None)
def SetControlPointAtIntersect(in_line_or_polygon_features=None, in_features=None):
    """SetControlPointAtIntersect_cartography(in_line_or_polygon_features, {in_features})

        Creates a control point at vertices that are shared by one or more
        line or polygon features. This tool is commonly used to synchronize
        boundary symbology on adjacent polygons.

     INPUTS:
      in_line_or_polygon_features (Feature Layer):
          The line or polygon feature layer.
      in_features {Feature Layer}:
          The feature layer with coincident features."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.SetControlPointAtIntersect_cartography(*gp_fixargs((in_line_or_polygon_features, in_features), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('SetControlPointByAngle_cartography', None)
def SetControlPointByAngle(in_features=None, maximum_angle=None):
    """SetControlPointByAngle_cartography(in_features, maximum_angle)

        Places a control point at vertices along a line or polygon outline
        where the angle created by a change in line direction is less than or
        equal to a specified maximum angle.

     INPUTS:
      in_features (Feature Layer):
          The feature layer containing line or polygon features.
      maximum_angle (Double):
          The angle used to determine whether or not a vertex along a line or
          polygon outline will be set as a control point. The angle value must
          be greater than zero and less than 180 decimal degrees."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.SetControlPointByAngle_cartography(*gp_fixargs((in_features, maximum_angle), True)))
        return retval
    except Exception as e:
        raise e


# Generalization toolset
@gptoolnotsupported
@gptooldoc('AggregatePoints_cartography', None)
def AggregatePoints(in_features=None, out_feature_class=None, aggregation_distance=None):
    """AggregatePoints_cartography(in_features, out_feature_class, aggregation_distance)

        Creates polygon features around clusters of proximate point features.

     INPUTS:
      in_features (Feature Layer):
          The input point features that will be assessed for proximity and
          clustering.
      aggregation_distance (Linear Unit):
          The distance between points that will be clustered.

     OUTPUTS:
      out_feature_class (Feature Class):
          The feature class created to hold the polygons that represent the
          point clusters."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.AggregatePoints_cartography(*gp_fixargs((in_features, out_feature_class, aggregation_distance), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('AggregatePolygons_cartography', None)
def AggregatePolygons(in_features=None, out_feature_class=None, aggregation_distance=None, minimum_area=None, minimum_hole_size=None, orthogonality_option=None, barrier_features=None, out_table=None):
    """AggregatePolygons_cartography(in_features, out_feature_class, aggregation_distance, {minimum_area}, {minimum_hole_size}, {orthogonality_option}, {barrier_features;barrier_features...}, {out_table})

        Combines polygons within a specified distance of each other into new
        polygons.

     INPUTS:
      in_features (Feature Layer):
          The polygon features to be aggregated. If this is a layer referencing
          a representation and shape overrides are present on the input
          features, the overridden shapes, not the feature shapes, will be
          considered in aggregation processing.
      aggregation_distance (Linear Unit):
          The distance to be satisfied between polygon boundaries for
          aggregation to happen. A distance must be specified, and it must be
          greater than zero. You can choose a preferred unit; the default is the
          feature unit.
      minimum_area {Areal Unit}:
          The minimum area for an aggregated polygon to be retained. The default
          value is zero, that is, to keep all polygons. You can specify a
          preferred unit; the default is the feature unit.
      minimum_hole_size {Areal Unit}:
          The minimum size of a polygon hole to be retained. The default value
          is zero, that is, to keep all polygon holes. You can specify a
          preferred unit; the default is the feature unit.
      orthogonality_option {Boolean}:
          Specifies the characteristic of the output features when constructing
          the aggregated boundaries.

          * NON_ORTHOGONAL-Organically shaped output features will be created.
          This is suitable for natural features, such as vegetation or soil
          polygons. This is the default.

          * ORTHOGONAL-Orthogonally shaped output features will be created. This
          option is suitable for preserving the geometric characteristic of
          anthropogenic input features, such as building footprints.
      barrier_features {Feature Layer}:
          The layers containing the line or polygon features that are
          aggregation barriers for input features. Features will not be
          aggregated across barrier features. Barrier features that are in
          geometric conflict with input features will be ignored.

     OUTPUTS:
      out_feature_class (Feature Class):
          The output feature class to be created.
      out_table {Table}:
          A one-to-many relationship table that links the aggregated polygons
          to their source polygon features. This table contains two fields,
          OUTPUT_FID and INPUT_FID, storing the aggregated feature IDs and their
          source feature IDs, respectively. Use this table to derive necessary
          attributes for the output features from their source features. The
          default name for this table is the name of the output feature class,
          appended with _tbl. The default path is the same as the output feature
          class. No table is created when this parameter is left blank."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.AggregatePolygons_cartography(*gp_fixargs((in_features, out_feature_class, aggregation_distance, minimum_area, minimum_hole_size, orthogonality_option, barrier_features, out_table), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('CollapseRoadDetail_cartography', None)
def CollapseRoadDetail(in_features=None, collapse_distance=None, output_feature_class=None):
    """CollapseRoadDetail_cartography(in_features, collapse_distance, output_feature_class)

        Collapses small, open configurations of road segments that interrupt
        the general trend of a road network, such as traffic circles, for
        example, and replaces them with a simplified depiction.Configurations
        are collapsed regardless of road class if the diameter
        across the open area is less than or equal to the Collapse Distance
        parameter. All uncollapsed roads from the input collection are copied
        to the output feature class.To learn more, see How Collapse Road
        Detail works.

     INPUTS:
      in_features (Feature Layer):
          The input features containing small enclosed road details, such as
          traffic circles, to be collapsed.
      collapse_distance (Linear Unit):
          The diameter of or distance across the road detail that is to be
          considered for collapse.

     OUTPUTS:
      output_feature_class (Feature Class):
          The output feature class containing the collapsed features-features
          that were modified to accommodate the collapse-and all unaffected
          features."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CollapseRoadDetail_cartography(*gp_fixargs((in_features, collapse_distance, output_feature_class), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('CreateCartographicPartitions_cartography', None)
def CreateCartographicPartitions(in_features=None, out_features=None, feature_count=None):
    """CreateCartographicPartitions_cartography(in_features;in_features..., out_features, feature_count)

        Creates a mesh of polygon features that cover the input feature
        class where each polygon encloses no more than a specified number of
        input features, determined by the density and distribution of the
        input features. The resulting partition feature class is ideally
        suited for the
        Cartographic Partitions geoprocessing environment setting. The
        Cartographic Partitions environment setting dictates to certain
        geoprocessing tools to load and process input features by partition.
        These tools operate contextually, meaning that multiple features,
        possibly from multiple themes, must be loaded simultaneously. Memory
        limitations are encountered with large datasets. Partitioning allows
        large datasets to be processed by these tools in portions
        sequentially.

     INPUTS:
      in_features (Feature Layer):
          The input feature classes or layers whose feature distribution and
          density dictate the size and arrangement of output polygons. The input
          features are typically destined for subsequent processing with other
          geoprocessing tools. Typically, the input features, when considered
          simultaneously, would exceed memory limitations of other tools, so
          partitions are created to subdivide inputs for processing.
      feature_count (Long):
          The ideal number of features to be enclosed by each polygon in the
          output feature class.  The recommended count is 50,000 features, which
          is the default value. The feature count cannot be lower than 500.

     OUTPUTS:
      out_features (Feature Class):
          The output polygon feature class of partitions, each of which encloses
          a manageable number of input features not exceeding the number
          specified by the feature_count parameter."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateCartographicPartitions_cartography(*gp_fixargs((in_features, out_features, feature_count), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('DelineateBuiltUpAreas_cartography', None)
def DelineateBuiltUpAreas(in_buildings=None, identifier_field=None, edge_features=None, grouping_distance=None, minimum_detail_size=None, out_feature_class=None, minimum_building_count=None):
    """DelineateBuiltUpAreas_cartography(in_buildings;in_buildings..., {identifier_field}, {edge_features;edge_features...}, grouping_distance, minimum_detail_size, out_feature_class, {minimum_building_count})

        Creates polygons to represent built-up areas by delineating densely
        clustered arrangements of buildings on small-scale maps.The
        boundaries-or edges-of the output polygons can be dictated by the
        location of other features such as roads or hydrology. Input buildings
        can be attributed to identify which can be replaced in maps by the
        built-up area polygons for a more generalized depiction.

     INPUTS:
      in_buildings (Feature Layer):
          The layers containing buildings whose density and arrangement are
          used to define appropriate output built-up polygons. Multiple building
          layers can be assessed simultaneously. Building features can be points
          or polygons.
      identifier_field {String}:
          A field on the input feature classes that will hold a status code
          indicating whether the input feature is part of the resulting built-up
          area . This field must be either short or long integer type and common
          in all input layers, if multiple input layers are used.

          * 0 = The building is not represented by an output built-up area
          polygon.

          * 1 = The building is represented by an output built-up area polygon
          and is within the resulting polygon.

          * 2 = The building is represented by an output built-up area polygon
          and is outside the resulting polygon.
      edge_features {Feature Layer}:
          The layers that can be used to define the edges of the built-up area
          polygons. Typically, these are roads, but other common examples are
          rivers, coastlines, or administrative areas. Built-up area polygons
          snap to an edge feature if one is generally aligned with the trend of
          the polygon edge and within the grouping distance away. Edge features
          can be lines or polygons.
      grouping_distance (Linear Unit):
          Buildings closer together than the grouping distance are considered
          collectively as candidates for representation by an output built-up
          area polygon. This distance is measured from the edges of polygon
          buildings and the centers of point buildings.
      minimum_detail_size (Linear Unit):
          Defines the relative degree of detail in the output built-up area
          polygons. This roughly translates to the minimum allowable diameter of
          a hole or cavity in the built-up area polygon. The actual size and
          shape of holes and cavities within the polygon is determined also by
          the arrangement of the input buildings, the grouping distance, and the
          presence of edge features, if they are used.
      minimum_building_count {Long}:
          The minimum number of buildings that must be collectively considered
          for representation by an output built-up area polygon. The default
          value is 4. The minimum building count must be greater than or equal
          to 0.

     OUTPUTS:
      out_feature_class (Feature Class):
          The output feature class containing built-up area polygons
          representing clustered arrangements of input buildings."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.DelineateBuiltUpAreas_cartography(*gp_fixargs((in_buildings, identifier_field, edge_features, grouping_distance, minimum_detail_size, out_feature_class, minimum_building_count), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('MergeDividedRoads_cartography', None)
def MergeDividedRoads(in_features=None, merge_field=None, merge_distance=None, out_features=None, out_displacement_features=None, character_field=None, out_table=None):
    """MergeDividedRoads_cartography(in_features, merge_field, merge_distance, out_features, {out_displacement_features}, {character_field}, {out_table})

        Generates single-line road features in place of matched pairs of
        divided road lanes.Matched pairs of roads or lanes are merged if they
        are the same road
        class, trend generally parallel to one another, and are within the
        merge distance apart. The road class is specified by the Merge Field
        parameter. All nonmerged roads from the input collection are copied to
        the output feature class. A warning is raised if the input features
        are not in a projected
        coordinate system. This tool relies on linear distance units, which
        will create unexpected results in an unprojected coordinate system. It
        is strongly suggested that you run this tool on data in a projected
        coordinate system to ensure valid results. An error is raised and the
        tool will not process if the coordinate system is missing or unknown.

     INPUTS:
      in_features (Feature Layer):
          The input linear road features that contain matched pairs of divided
          road lanes that should be merged together to a single output line
          feature.
      merge_field (Field):
          The field that contains road classification information. Only
          parallel, proximate roads of equal classification will be merged. A
          value of 0 (zero) locks a feature to prevent it from participating in
          merging.
      merge_distance (Linear Unit):
          The minimum distance apart, in the specified units, for equal-class,
          relatively parallel road features to be merged. This distance must be
          greater than zero. If the units are in points, millimeters,
          centimeters, or inches, the value is considered as page units and
          takes into account the reference scale.
      character_field {Field}:
          Specify a numeric field that indicate the character of road segments,
          independent of their road classification. These values help the tool
          to refine the assessment of candidate feature pairs for merging. Use
          this parameter in unusual or complex road networks to improve the
          quality of the output. Where there are null values (or if this
          parameter is not specified at all), the road character (and merge
          candidacy) is based only on the shapes and arrangement of features.
          Use value 999 to lock features from participating in a merge at
          all.Field values are assessed as follows:

          * 0-Traffic circles or roundabouts

          * 1-Carriageways, boulevards, dual-lane highways, or other parallel
          trending roads

          * 2-On- or off-ramps, highway intersection connectors

          * 999-Features that should not merge

     OUTPUTS:
      out_features (Feature Class):
          The output feature class containing single-line merged road features
          and all unmerged road features.
      out_displacement_features {Feature Class}:
          The output polygon features containing the degree and direction of
          road displacement.
      out_table {Table}:
          A many-to-many relationship table that links the merged road features
          to their source features. This table contains two fields, OUTPUT_FID
          and INPUT_FID, storing the merged feature IDs and their source feature
          IDs, respectively. Use this table to derive necessary attributes for
          the output features from their source features. No table is created
          when this parameter is left blank."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.MergeDividedRoads_cartography(*gp_fixargs((in_features, merge_field, merge_distance, out_features, out_displacement_features, character_field, out_table), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('SimplifyBuilding_cartography', None)
def SimplifyBuilding(in_features=None, out_feature_class=None, simplification_tolerance=None, minimum_area=None, conflict_option=None):
    """SimplifyBuilding_cartography(in_features, out_feature_class, simplification_tolerance, {minimum_area}, {conflict_option})

        Simplifies the boundary or footprint of building polygons while
        maintaining their essential shape and size.

     INPUTS:
      in_features (Feature Layer):
          The building polygons to be simplified.
      simplification_tolerance (Linear Unit):
          Sets the tolerance for building simplification. A tolerance must be
          specified, and it must be greater than zero. You can choose a
          preferred unit; the default is the feature unit.
      minimum_area {Areal Unit}:
          Sets the minimum area for a simplified building to be retained in
          feature units. The default value is zero, that is, to keep all
          buildings. You can specify a preferred unit; the default is the
          feature unit.
      conflict_option {Boolean}:
          Specifies whether or not to check for potential conflicts, that is,
          overlapping or touching, among buildings. A field named SimBldFlag is
          added to the output to store conflict flags. A value of 0 means no
          conflict; a value of 1 means conflict.

          * NO_CHECK-Specifies not to check for potential conflicts; the
          resulting buildings may overlap. This is the default.

          * CHECK_CONFLICTS-Specifies to check for potential conflicts; the
          conflicting buildings will be flagged.

     OUTPUTS:
      out_feature_class (Feature Class):
          The output feature class to be created."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.SimplifyBuilding_cartography(*gp_fixargs((in_features, out_feature_class, simplification_tolerance, minimum_area, conflict_option), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('SimplifyLine_cartography', None)
def SimplifyLine(in_features=None, out_feature_class=None, algorithm=None, tolerance=None, error_resolving_option=None, collapsed_point_option=None, error_checking_option=None, in_barriers=None):
    """SimplifyLine_cartography(in_features, out_feature_class, algorithm, tolerance, {error_resolving_option}, {collapsed_point_option}, {error_checking_option}, {in_barriers;in_barriers...})

        Simplifies lines by removing relatively extraneous vertices while
        preserving essential shape.

     INPUTS:
      in_features (Feature Layer):
          The input line features to be simplified.
      algorithm (String):
          Specifies the line simplification algorithm.

          * POINT_REMOVE-Retains critical points that preserve the essential
          shape of a line and removes all other points. This is the default.

          * BEND_SIMPLIFY-Retains the critical bends and removes extraneous
          bends from a line.

          * WEIGHTED_AREA-Retains vertices that form triangles of effective
          area.
      tolerance (Linear Unit):
          The tolerance determines the degree of simplification. You can choose
          a preferred unit; otherwise, the units of the input will be used.
          MinSimpTol and MaxSimpTol fields are added to the output to store the
          tolerance that was used when processing occurred.

          * For the POINT_REMOVE algorithm, the tolerance is the maximum
          allowable perpendicular distance between each vertex and the new line
          created.

          * For the BEND_SIMPLIFY algorithm, the tolerance is the diameter of a
          circle that approximates a significant bend.

          * For the WEIGHTED_AREA algorithm, the square of the tolerance is the
          area of a significant  triangle defined by three adjacent vertices.
      error_resolving_option {Boolean}:
          This is a legacy parameter that is no longer used. It was formerly
          used to dictate how topological errors, possibly introduced during
          processing, were handled. This parameter is still included in the
          tool's syntax for compatibility in scripts and models but is hidden
          from the tool's dialog box.
      collapsed_point_option {Boolean}:
          Dictate whether to create an output point feature class to store the
          endpoints of any lines that are smaller than the spatial tolerance.
          The point output is derived; it will use the same name and location as
          the out_feature_class but with a _Pnt suffix.

          * KEEP_COLLAPSED_POINTS-Record the endpoints of zero-length lines in a
          derived output point feature class. This is the default.

          * NO_KEEP-Do not create a derived output point feature class.
      error_checking_option {Boolean}:
          This is a legacy parameter that is no longer used. It was formerly
          used to dictate how topological errors, possibly introduced during
          processing, were handled. This parameter is still included in the
          tool's syntax for compatibility in scripts and models but is hidden
          from the tool's dialog box.
      in_barriers {Feature Layer}:
          Inputs containing features to act as barriers for simplification.
          Resulting simplified lines will not touch or cross barrier features.
          For example, when simplifying contour lines, spot height features
          input as barriers ensure that the simplified contour lines will not
          simplify across these points. The output will not violate the
          elevation as described by measured spot heights.

     OUTPUTS:
      out_feature_class (Feature Class):
          The simplified output line feature class. It contains all fields
          present in the input feature class. The output line feature class is
          topologically correct. The tool does not introduce topology errors,
          but topological errors in the input data are flagged in the output
          line feature class. The output feature class includes two additional
          fields: InLine_FID and SimLnFlag to contain the input feature IDs and
          the input topological errors, respectively. A SimLnFlag value of 1
          indicates an input topological error is present; 0 (zero) indicates
          that no input error is present."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.SimplifyLine_cartography(*gp_fixargs((in_features, out_feature_class, algorithm, tolerance, error_resolving_option, collapsed_point_option, error_checking_option, in_barriers), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('SimplifyPolygon_cartography', None)
def SimplifyPolygon(in_features=None, out_feature_class=None, algorithm=None, tolerance=None, minimum_area=None, error_option=None, collapsed_point_option=None, in_barriers=None):
    """SimplifyPolygon_cartography(in_features, out_feature_class, algorithm, tolerance, {minimum_area}, {error_option}, {collapsed_point_option}, {in_barriers;in_barriers...})

        Simplifies polygon outlines by removing relatively extraneous vertices
        while preserving essential shape.

     INPUTS:
      in_features (Feature Layer):
          The input polygon features to be simplified.
      algorithm (String):
          Specifies the polygon simplification algorithm.

          * POINT_REMOVE-Retains critical points that preserve the essential
          shape of a polygon outline and removes all other points. This is the
          default.

          * BEND_SIMPLIFY-Retains the critical bends and removes extraneous
          bends from a polygon.

          * WEIGHTED_AREA-Retains vertices that form triangles of the effective
          area.
      tolerance (Linear Unit):
          The tolerance determines the degree of simplification. You can choose
          a preferred unit; otherwise, units of the input will be used.
          MinSimpTol and MaxSimpTol fields are added to the output to store the
          tolerance that was used when processing occurred.

          * For the POINT_REMOVE algorithm, the tolerance is the maximum
          allowable perpendicular distance between each vertex and the new line
          created.

          * For the BEND_SIMPLIFY algorithm, the tolerance is the diameter of a
          circle that approximates a significant bend.

          * For the WEIGHTED_AREA algorithm, the square of the tolerance is the
          area of a significant  triangle defined by three adjacent vertices.
      minimum_area {Areal Unit}:
          Sets the minimum area for a polygon to be retained. The default value
          is zero, that is, to keep all polygons. You can choose a preferred
          unit for the specified value; otherwise, units of the input will be
          used.
      error_option {String}:
          This is a legacy parameter that is no longer used. It was formerly
          used to dictate how topological errors possibly introduced during
          processing were handled. This parameter is still included in the
          tool's syntax for compatibility in scripts and model but is hidden
          from the tool's dialog box.
      collapsed_point_option {Boolean}:
          Dictate whether to create an output point feature class to store the
          centers of any polygons removed because they are smaller than the
          minimum_area parameter. The point output is derived; it will be named
          the same as the polygon output feature class you specify with the
          out_feature_class parameter but with a _Pnt suffix, located in the
          same folder.

          * KEEP_COLLAPSED_POINTS-Record the centers of polygons that are
          removed because they are below the minimum area in a derived output
          point feature class. This is the default.

          * NO_KEEP-Do not create a derived output point feature class.
      in_barriers {Feature Layer}:
          Inputs containing features to act as barriers for simplification.
          Resulting simplified polygons will not touch or cross barrier
          features. For example, when simplifying forested areas, the resulting
          simplified forest polygons do not cross road features defined as
          barriers.

     OUTPUTS:
      out_feature_class (Feature Class):
          The simplified output polygon feature class. It contains all fields
          present in the input feature class. The output polygon feature class
          is topologically correct. The tool does not introduce topology errors,
          but topological errors in the input data are flagged in the output
          polygon feature class. The output feature class includes two
          additional fields: InPoly_FID and SimPgnFlag to contain the input
          feature IDs and the input topological errors, respectively. A
          SimPgnFlag value of 1 indicates an input topological error is present;
          0 (zero) indicates that no input error is present."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.SimplifyPolygon_cartography(*gp_fixargs((in_features, out_feature_class, algorithm, tolerance, minimum_area, error_option, collapsed_point_option, in_barriers), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('SmoothLine_cartography', None)
def SmoothLine(in_features=None, out_feature_class=None, algorithm=None, tolerance=None, endpoint_option=None, error_option=None):
    """SmoothLine_cartography(in_features, out_feature_class, algorithm, tolerance, {endpoint_option}, {error_option})

        Smooths sharp angles in lines to improve aesthetic or cartographic
        quality.Learn more about how the Smooth Line and Smooth Polygon tools
        work.

     INPUTS:
      in_features (Feature Layer):
          The line features to be smoothed.
      algorithm (String):
          Specifies the smoothing algorithm.

          * PAEK-Acronym for Polynomial Approximation with Exponential Kernel.
          It calculates a smoothed line that will not pass through the input
          line vertices. This is the default.

          * BEZIER_INTERPOLATION-Fits Bezier curves between vertices. The
          resulting line passes through the vertices of the input line. This
          algorithm does not require a tolerance. Bezier curves will be
          approximated in shapefile output.
      tolerance (Linear Unit):
          Sets a tolerance used by the PAEK algorithm. A tolerance must be
          specified, and it must be greater than zero. You can choose a
          preferred unit; the default is the feature unit. You must enter a 0 as
          a placeholder when using the BEZIER_INTERPOLATION smoothing algorithm.
      endpoint_option {Boolean}:
          Specifies whether to preserve the endpoints for closed lines. This
          option works with the PAEK algorithm only.

          * FIXED_CLOSED_ENDPOINT-Preserves the endpoint of a closed line. This
          is the default.

          * NO_FIXED-Smooths through the endpoint of a closed line.
      error_option {String}:
          Specifies how the topological errors (possibly introduced in the
          process, such as line crossing) will be handled.

          * NO_CHECK-Specifies not to check for topological errors. This is the
          default.

          * FLAG_ERRORS-Specifies to flag topological errors, if any are found.

     OUTPUTS:
      out_feature_class (Feature Class):
          The output feature class to be created."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.SmoothLine_cartography(*gp_fixargs((in_features, out_feature_class, algorithm, tolerance, endpoint_option, error_option), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('SmoothPolygon_cartography', None)
def SmoothPolygon(in_features=None, out_feature_class=None, algorithm=None, tolerance=None, endpoint_option=None, error_option=None):
    """SmoothPolygon_cartography(in_features, out_feature_class, algorithm, tolerance, {endpoint_option}, {error_option})

        Smooths sharp angles in polygon outlines to improve aesthetic or
        cartographic quality.Learn more about how the Smooth Line and Smooth
        Polygon tools work.

     INPUTS:
      in_features (Feature Layer):
          The polygon features to be smoothed.
      algorithm (String):
          Specifies the smoothing algorithm.

          * PAEK-Acronym for Polynomial Approximation with Exponential Kernel.
          It calculates a smoothed polygon that will not pass through the input
          polygon vertices. This is the default.

          * BEZIER_INTERPOLATION-Fits Bezier curves between vertices. The
          resulting polygon passes through the vertices of input polygons. This
          algorithm does not require a tolerance. Bezier curves will be
          approximated in shapefile output.
      tolerance (Linear Unit):
          Sets a tolerance used by the PAEK algorithm. A tolerance must be
          specified, and it must be greater than zero. You can specify a
          preferred unit; the default is the feature unit. You must enter a 0 as
          a placeholder when using the BEZIER_INTERPOLATION smoothing algorithm.
      endpoint_option {Boolean}:
          Specifies whether or not to preserve the endpoints for isolated
          polygon rings. This option works with the PAEK algorithm only.

          * FIXED_ENDPOINT-Preserves the endpoint of an isolated polygon ring.
          This is the default.

          * NO_FIXED-Smooths through the endpoint of an isolated polygon ring.
      error_option {String}:
          Specifies how the topological errors (possibly introduced in the
          process, such as line crossing or overlapping) will be handled.

          * NO_CHECK-Specifies not to check for topological errors. This is the
          default.

          * FLAG_ERRORS-Specifies to flag topological errors, if any are found.

     OUTPUTS:
      out_feature_class (Feature Class):
          The output polygon feature class to be created."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.SmoothPolygon_cartography(*gp_fixargs((in_features, out_feature_class, algorithm, tolerance, endpoint_option, error_option), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('ThinRoadNetwork_cartography', None)
def ThinRoadNetwork(in_features=None, minimum_length=None, invisibility_field=None, hierarchy_field=None):
    """ThinRoadNetwork_cartography(in_features;in_features..., minimum_length, invisibility_field, hierarchy_field)

        Generates a simplified road network that retains connectivity and
        general character for display at a smaller scale.This tool does not
        generate new output. It assigns values in
        Invisibility Field in the input feature classes to identify features
        that are extraneous and can be removed from view to result in a
        simplified, yet representative, collection of roads. No feature
        geometry is altered or deleted.Features are not actually deleted by
        Thin Road Network. To actually
        remove features, consider using the Trim Line tool.The resulting
        simplified road collection is determined by feature
        significance, importance, and density. Segments that participate in
        very long itineraries across the extent of the data are more
        significant than those required only for local travel. Road
        classification, or importance, is specified by the Hierarchy Field
        parameter. The density of the resulting street network is determined
        by the Minimum Length parameter, which corresponds to the shortest
        segment that is visually sensible to show at scale. Learn more about
        how Thin Road Network works and see a table of
        recommended minimum length values to use as a starting point. A
        warning is raised if the input features are not in a projected
        coordinate system. This tool relies on linear distance units, which
        will create unexpected results in an unprojected coordinate system. It
        is strongly suggested that you run this tool on data in a projected
        coordinate system to ensure valid results. An error is raised and the
        tool will not process if the coordinate system is missing or unknown.

     INPUTS:
      in_features (Feature Layer):
          The input linear roads that should be thinned to create a simplified
          collection for display at smaller scales.
      minimum_length (Linear Unit):
          An indication of the shortest road segment that is sensible to
          display at the output scale. This controls the resolution, or density,
          of the resulting road collection. If the units are in point,
          millimeters, centimeters, or inches, the value is considered in page
          units and the reference scale is taken into account.
      invisibility_field (String):
          The field that stores the results of the tool. Features that
          participate in the resulting simplified road collection have a value
          of 0 (zero). Those that are extraneous have a value of 1. A layer
          definition query can be used to display the resulting road collection.
          This field must be present and named the same for each input feature
          class.
      hierarchy_field (String):
          The field that contains hierarchical ranking of feature importance,
          where 1 is very important and larger integers reflect decreasing
          importance. A value of 0 forces the feature to remain visible in the
          output collection. This field must be present and named the same for
          each input feature class. Hierarchy values equal to NULL are not
          accepted and will raise an error."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ThinRoadNetwork_cartography(*gp_fixargs((in_features, minimum_length, invisibility_field, hierarchy_field), True)))
        return retval
    except Exception as e:
        raise e


# Map Series toolset
@gptoolnotsupported
@gptooldoc('GridIndexFeatures_cartography', None)
def GridIndexFeatures(out_feature_class=None, in_features=None, intersect_feature=None, use_page_unit=None, scale=None, polygon_width=None, polygon_height=None, origin_coord=None, number_rows=None, number_columns=None, starting_page_number=None, label_from_origin=None):
    """GridIndexFeatures_cartography(out_feature_class, {in_features;in_features...}, {intersect_feature}, {use_page_unit}, {scale}, {polygon_width}, {polygon_height}, {origin_coord}, {number_rows}, {number_columns}, {starting_page_number}, {label_from_origin})

        Creates a grid of rectangular polygon features that can be used as an
        index to specify pages for a map book using Map Series. A grid can be
        created that only includes polygon features that intersect another
        feature layer.

     INPUTS:
      in_features {Feature Layer / Raster Layer}:
          Input features can be used to define the extent of the polygon grid
          that is created.
      intersect_feature {Boolean}:
          Limits the output grid feature class to only areas that intersect
          input feature layers or datasets. When input features are specified,
          the default value is INTERSECTFEATURE. The intersection of input
          features will be used to create index features.

          * INTERSECTFEATURE-Limits the output grid feature class to only areas
          that intersect input feature layers or datasets.

          * NO_INTERSECTFEATURE-Output grid feature class is created using
          specified coordinates, rows, and columns.
      use_page_unit {Boolean}:
          Indicates whether index polygon size input is in page units. The
          default is NO_USEPAGEUNIT. The tool uses map units by default.

          * USEPAGEUNIT-Index polygon height and width are calculated in page
          units.

          * NO_USEPAGEUNIT-Index polygon height and width are calculated in map
          units.
      scale {Long}:
          Scale must be specified if the index polygon height and width are to
          be calculated in page units. If the tool is being used outside of an
          active ArcGIS Pro session the default scale value is 1.
      polygon_width {Linear Unit}:
          Width of the index polygon specified in either map or page units. If
          page units are being used the default value is 1 inch. If map units
          are being used the default is 1 degree.
      polygon_height {Linear Unit}:
          Height of the index polygon specified in either map or page units. If
          page units are being used the default value is 1 inch. If map units
          are being used the default is 1 degree.
      origin_coord {Point}:
          Coordinate for the lower left origin of the output grid feature
          class. If input features are specified the default value is determined
          by the extent of the union of extents for these features. If there are
          no input features specified, the default coordinates are 0 and 0.
      number_rows {Long}:
          Number of rows to create in the y direction from the point of origin.
          The default is 10.
      number_columns {Long}:
          Number of columns to create in the x direction from the point of
          origin. The default is 10.
      starting_page_number {Long}:
          Each grid index feature is assigned a sequential page number starting
          with a specified starting page number. The default is 1.
      label_from_origin {Boolean}:
          Page numbers (labels) starting with the specified starting page number
          (default is 1) begins with the cell in the lower left corner of the
          output grid. The default is NO_LABELFROMORIGIN.

          * LABELFROMORIGIN-Page numbers (labels) starting with the specified
          starting page number (default is 1) begins with the polygon feature in
          the lower left corner of the output grid.

          * NO_LABELFROMORIGIN-Page numbers (labels) starting with the specified
          starting page number (default is 1) begins with the cell in the upper
          left corner of the output grid.

     OUTPUTS:
      out_feature_class (Feature Class):
          Resulting feature class of polygon index features.The coordinate
          system of the output feature class is determined in
          this order.

          * If a coordinate system is specified by the Output Coordinate System
          variable in the Environment Settings the output feature class will use
          this coordinate system.

          * If a coordinate system is not specified by the Output Coordinate
          System the output feature class will use the coordinate system of the
          active map (ArcGIS Pro is open).

          * If a coordinate system is not specified by the Output Coordinate
          System and there is no active map (ArcGIS Pro is not open) the output
          feature class will use the coordinate system of the first input
          feature.

          * If a coordinate system is not specified by the Output Coordinate
          System, there is no active map (ArcGIS Pro is not open) and there is
          no specified input features the coordinate system of the output
          feature class will be unknown."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.GridIndexFeatures_cartography(*gp_fixargs((out_feature_class, in_features, intersect_feature, use_page_unit, scale, polygon_width, polygon_height, origin_coord, number_rows, number_columns, starting_page_number, label_from_origin), True)))
        return retval
    except Exception as e:
        raise e


# Masking toolset
@gptoolnotsupported
@gptooldoc('CulDeSacMasks_cartography', None)
def CulDeSacMasks(input_layer=None, output_fc=None, reference_scale=None, spatial_reference=None, margin=None, attributes=None):
    """CulDeSacMasks_cartography(input_layer, output_fc, reference_scale, spatial_reference, margin, attributes)

        Creates a feature class of polygon masks from a symbolized input line
        layer.

     INPUTS:
      input_layer (Layer):
          Input line layer from which to create masks.
      reference_scale (Double):
          The reference scale used for calculating the masking geometry when
          masks are specified in page units. This is typically the reference
          scale of the map.
      spatial_reference (Spatial Reference):
          The spatial reference for which the masking polygons will be created.
          This is not the spatial reference that will be assigned to the output
          feature class. It is the spatial reference of the map in which the
          masking polygons will be used since the position of symbology may
          change when features are projected.
      margin (Linear Unit):
          The space in page units surrounding the symbolized input features used
          to create the mask polygon. Typically, masking polygons are created
          with a small margin around the symbol to improve visual appearance.
          Margin values are specified in either page units or map units. Most of
          the time you will want to specify your margin distance value in page
          units.The margin cannot be negative.
      attributes (String):
          Determines which attributes will be transferred from the input
          features to the output features.

          * ONLY_FID-Only the FID field from the input features will be
          transferred to the output features. This is the default.

          * NO_FID-All the attributes except the FID from the input features
          will be transferred to the output features.

          * ALL-All the attributes from the input features will be transferred
          to the output features.

     OUTPUTS:
      output_fc (Feature Class):
          The feature class that will contain the mask features."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CulDeSacMasks_cartography(*gp_fixargs((input_layer, output_fc, reference_scale, spatial_reference, margin, attributes), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('FeatureOutlineMasks_cartography', None)
def FeatureOutlineMasks(input_layer=None, output_fc=None, reference_scale=None, spatial_reference=None, margin=None, method=None, mask_for_non_placed_anno=None, attributes=None):
    """FeatureOutlineMasks_cartography(input_layer, output_fc, reference_scale, spatial_reference, margin, method, mask_for_non_placed_anno, attributes)

        Creates mask polygons at a specified distance and shape around the
        symbolized features in the input layer.

     INPUTS:
      input_layer (Layer):
          The symbolized input layer from which the masks will be created.
      reference_scale (Double):
          The reference scale used for calculating the masking geometry when
          masks are specified in page units. This is typically the reference
          scale of the map.
      spatial_reference (Spatial Reference):
          The spatial reference for which the masking polygons will be created.
          This is not the spatial reference that will be assigned to the output
          feature class. It is the spatial reference of the map in which the
          masking polygons will be used since the position of symbology may
          change when features are projected.
      margin (Linear Unit):
          The space in page units surrounding the symbolized input features used
          to create the mask polygon. Typically, masking polygons are created
          with a small margin around the symbol to improve visual appearance.
          Margin values are specified in either page units or map units. Most of
          the time you will want to specify your margin distance value in page
          units.The margin cannot be negative.
      method (String):
          The type of masking geometry created. There are four types:

          * BOX-A polygon representing the extent of the symbolized feature.

          * CONVEX_HULL-The convex hull of the symbolized geometry of the
          feature. This is the default.

          * EXACT_SIMPLIFIED-A generalized polygon representing the exact shape
          of the symbolized feature. Polygons created with this method will have
          a significantly smaller number of vertices compared to polygons
          created with the EXACT method.

          * EXACT-A polygon representing the exact shape of the symbolized
          feature.
      mask_for_non_placed_anno (String):
          Specifies whether to create masks for unplaced annotation. This option
          is only used when masking geodatabase annotation layers.

          * ALL_FEATURES-Creates masks for all the annotation features.

          * ONLY_PLACED-Only creates masks for features with a status of placed.
      attributes (String):
          Determines which attributes will be transferred from the input
          features to the output features.

          * ONLY_FID-Only the FID field from the input features will be
          transferred to the output features. This is the default.

          * NO_FID-All the attributes except the FID from the input features
          will be transferred to the output features.

          * ALL-All the attributes from the input features will be transferred
          to the output features.

     OUTPUTS:
      output_fc (Feature Class):
          The feature class that will contain the mask features."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.FeatureOutlineMasks_cartography(*gp_fixargs((input_layer, output_fc, reference_scale, spatial_reference, margin, method, mask_for_non_placed_anno, attributes), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('IntersectingLayersMasks_cartography', None)
def IntersectingLayersMasks(masking_layer=None, masked_layer=None, output_fc=None, reference_scale=None, spatial_reference=None, margin=None, method=None, mask_for_non_placed_anno=None, attributes=None):
    """IntersectingLayersMasks_cartography(masking_layer, masked_layer, output_fc, reference_scale, spatial_reference, margin, method, mask_for_non_placed_anno, attributes)

        Creates masking polygons at a specified shape and size at the
        intersection of two symbolized input layers: the masking layer and the
        masked layer.

     INPUTS:
      masking_layer (Layer):
          The symbolized input layer, which will be intersected with the masked
          layer to create masking polygons. This is the layer that will be more
          prominently displayed when masking is applied to the masked layer.
      masked_layer (Layer):
          The symbolized input layer to be masked. This is the layer that will
          be obscured by the masking polygons.
      reference_scale (Double):
          The reference scale used for calculating the masking geometry when
          masks are specified in page units. This is typically the reference
          scale of the map.
      spatial_reference (Spatial Reference):
          The spatial reference for which the masking polygons will be created.
          This is not the spatial reference that will be assigned to the output
          feature class. It is the spatial reference of the map in which the
          masking polygons will be used since the position of symbology may
          change when features are projected.
      margin (Linear Unit):
          The space in page units surrounding the symbolized input features used
          to create the mask polygon. Typically, masking polygons are created
          with a small margin around the symbol to improve visual appearance.
          Margin values are specified in either page units or map units. Most of
          the time you will want to specify your margin distance value in page
          units.The margin cannot be negative.
      method (String):
          The type of masking geometry created. There are four types:

          * BOX-A polygon representing the extent of the symbolized feature.

          * CONVEX_HULL-The convex hull of the symbolized geometry of the
          feature. This is the default.

          * EXACT_SIMPLIFIED-A generalized polygon representing the exact shape
          of the symbolized feature. Polygons created with this method will have
          a significantly smaller number of vertices compared to polygons
          created with the EXACT method.

          * EXACT-A polygon representing the exact shape of the symbolized
          feature.
      mask_for_non_placed_anno (String):
          Specifies whether to create masks for unplaced annotation. This option
          is only used when masking geodatabase annotation layers.

          * ALL_FEATURES-Creates masks for all the annotation features.

          * ONLY_PLACED-Only creates masks for features with a status of placed.
      attributes (String):
          Determines which attributes will be transferred from the input
          features to the output features.

          * ONLY_FID-Only the FID field from the input features will be
          transferred to the output features. This is the default.

          * NO_FID-All the attributes except the FID from the input features
          will be transferred to the output features.

          * ALL-All the attributes from the input features will be transferred
          to the output features.

     OUTPUTS:
      output_fc (Feature Class):
          The feature class that will contain the mask features."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.IntersectingLayersMasks_cartography(*gp_fixargs((masking_layer, masked_layer, output_fc, reference_scale, spatial_reference, margin, method, mask_for_non_placed_anno, attributes), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject, gptoolnotsupported