# -*- coding: utf-8 -*-
#COPYRIGHT 2017 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
r"""ArcGIS Pipeline Referencing is a data management application suite
that provides an integrated framework for maintaining pipeline data
with linear referencing systems. The Location Referencing toolbox
provides essential tools for regenerating routes, managing calibration
points and events, and executing quality control checks."""
__all__ = ['AppendEvents', 'ApplyEventBehaviors', 'CreateLRS', 'CreateLRSEvent',
           'CreateLRSEventFromExistingDataset', 'CreateLRSFromExistingDataset',
           'CreateLRSNetwork', 'CreateLRSNetworkFromExistingDataset',
           'CreateUPDMGeodatabase', 'DeriveEventMeasures',
           'DisableReferentFields', 'EnableReferentFields', 'GenerateEvents',
           'GenerateRoutes', 'ModifyEventBehaviorRules', 'ModifyLRS',
           'ModifyLRSEvent', 'ModifyLRSNetwork', 'OverlayEvents',
           'RemoveLRSEntity', 'TranslateEventMeasures']
__alias__ = 'locref'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs, gptoolnotsupported
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Tools
@gptoolnotsupported
@gptooldoc('AppendEvents_locref', None)
def AppendEvents(in_dataset=None, in_target_event=None, field_mapping=None, load_type=None, generate_event_ids=None, generate_shapes=None):
    """AppendEvents_locref(in_dataset, in_target_event, field_mapping, {load_type}, {generate_event_ids}, {generate_shapes})

        Appends event records from a table, layer, or feature class into an
        existing ArcGIS Pipeline Referencing event feature class.

     INPUTS:
      in_dataset (Table View):
          The source event records to append.
      in_target_event (Feature Layer):
          The Pipeline Referencing event layer or feature class into which the
          source event records append.
      field_mapping (Field Mappings):
          Controls how the attribute information in the fields of the
          in_dataset is transferred to the in_target_event. Because the data of
          the in_dataset is appended into an existing event
          that has a predefined schema (field definitions), fields cannot be
          added or removed from the target dataset. While you can set merge
          rules for each output field, the tool ignores those rules.The ArcPy
          FieldMappings class can be used to define this parameter.
      load_type {String}:
          Determines how appended events with measure or temporality overlaps
          with identical Event IDs as Target Event records are loaded into the
          event feature class.

          * ADD-Appends the Input Event records to the Target Event. No changes
          are made to Target Event records.

          * RETIRE_OVERLAPS-Appends the Input Event records to the Target Event
          and retires any records in the Target Event with measure or
          temporality overlaps as the appended events. If the appended event
          eclipses the Target Event, the Target Event will be deleted. This
          option should only be used for linear events.

          * RETIRE_BY_EVENT_ID-Appends the Input Event records to the Target
          Event and retires any records in the Target Event with the same Event
          ID and temporality overlaps as the appended events. If the appended
          event eclipses a Target Event with the same Event ID, the Target Event
          will be deleted.

          * REPLACE_BY_EVENT_ID-Appends the Input Event records to the Target
          Event and deletes any records in the Target Event with the same Event
          ID as the appended events.
      generate_event_ids {Boolean}:
          Indicates whether to generate Event IDs for in_dataset records being
          appended. Generation of Events IDs will only be applied to in_dataset
          records with a Null value for the Event ID field.

          * GENERATE_EVENT_IDS-Generates Event IDs of thein_dataset records
          being appended.

          * NO_GENERATE_EVENT_IDS-Does not generate Event IDs of the in_dataset
          records being appended. This is the default.
      generate_shapes {Boolean}:
          Indicates whether to regenerate the shapes for the records being
          appended. This parameter is only active when the in_dataset is a
          feature layer or feature class.

          * GENERATE_SHAPES-Regenerates the shapes of the input event features.
          This is the default.

          * NO_SHAPES-Does not regenerate the shapes of the input event
          features."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.AppendEvents_locref(*gp_fixargs((in_dataset, in_target_event, field_mapping, load_type, generate_event_ids, generate_shapes), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('ApplyEventBehaviors_locref', None)
def ApplyEventBehaviors(in_route_features=None):
    """ApplyEventBehaviors_locref(in_route_features)

        Updates the event locations for all event feature classes registered
        with the input network based on the route edit performed.The events
        get updated as per the event behavior configured. To learn
        more about event behavior and configuring event behavior, see  Event
        behaviors.

     INPUTS:
      in_route_features (Feature Layer):
          The ALRS Network for which event locations will be updated. This must
          be a network feature class."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ApplyEventBehaviors_locref(*gp_fixargs((in_route_features,), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('CreateUPDMGeodatabase_locref', None)
def CreateUPDMGeodatabase(output_gdb=None, spatial_reference=None, measure_unit=None, xy_tolerance=None, z_tolerance=None, event_registration_network=None, register_p_pipesystem=None):
    """CreateUPDMGeodatabase_locref(output_gdb, spatial_reference, {measure_unit}, {xy_tolerance}, {z_tolerance}, {event_registration_network}, {register_p_pipesystem})

        Populates an existing file or multiuser geodatabase with all feature
        datasets, feature classes, tables, relationship classes, and domains
        needed for the Utility and Pipeline Data Model (UPDM) and creates an
        ALRS with events registered to the chosen network.

     INPUTS:
      output_gdb (Workspace):
          The file or multiuser geodatabase where the UPDM ALRS will be
          created.
      spatial_reference (Spatial Reference):
          The spatial reference for feature datasets and feature classes within
          the geodatabase.
      measure_unit {String}:
          The unit of measure for the M values. This is the unit of measure
          that will be used for the LRS Networks created by the tool.

          * CENTIMETERS-Centimeters is the unit of measure.

          * DECIMETERS-Decimeters is the unit of measure.

          * FEET-Feet is the unit of measure.

          * INCHES-Inches is the unit of measure.

          * KILOMETERS-Kilometers is the unit of measure.

          * METERS-Meters is the unit of measure.

          * MILES-Miles is the unit of measure.

          * MILLIMETERS-Millimeters is the unit of measure.

          * NAUTICAL_MILES-Nautical Miles is the unit of measure.

          * YARDS-Yards is the unit of measure.
      xy_tolerance {Double}:
          The XY tolerance for feature datasets and feature classes within the
          geodatabase. XY resolution will be one-tenth the XY tolerance. M
          tolerance will be equal to the XY tolerance converted to the selected
          M unit of measure. M resolution will be one-tenth the M tolerance.
      z_tolerance {Double}:
          The Z tolerance for feature datasets and feature classes within the
          geodatabase. Z resolution will be one-tenth the Z tolerance.
      event_registration_network {String}:
          The network to which events will be registered. Choose Engineering if
          your events will be registered to the Engineering network and will
          support lines and events that span routes. Choose Continuous if your
          events will be registered to the Continuous network and linear events
          do not need to span routes.

          * CONTINUOUS-Events will be registered to the Continuous network.

          * ENGINEERING-Events will be registered to the Engineering network.
      register_p_pipesystem {Boolean}:
          Determines whether all event feature classes in the network will be
          updated.

          * TRUE-Feature classes within P_PipeSystem will be registered as
          events.

          * FALSE-Feature classes within P_PipeSystem will not be registered as
          events. This is the default."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateUPDMGeodatabase_locref(*gp_fixargs((output_gdb, spatial_reference, measure_unit, xy_tolerance, z_tolerance, event_registration_network, register_p_pipesystem), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('DeriveEventMeasures_locref', None)
def DeriveEventMeasures(in_route_features=None, update_all_events=None, event_layers=None):
    """DeriveEventMeasures_locref(in_route_features, {update_all_events}, {event_layers;event_layers...})

        Populates and updates the Derived RouteID and measure values on point
        and line events with those fields configured and enabled.

     INPUTS:
      in_route_features (Feature Layer):
          The LRS Network containing the events with Derived RouteID and
          measure fields configured.
      update_all_events {Boolean}:
          Determines whether all event feature classes in the network will be
          updated.

          * UPDATE_ALL-Updates all events in the network selected in
          in_route_features. This is the default.

          * UPDATE_SOME-Does not update all events in the network selected in
          the in_route_features. Allows the user to select individual event
          layers in the event_layers parameter below.
      event_layers {Feature Layer}:
          The event layers that will have Derived RouteID and measure fields
          updated."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.DeriveEventMeasures_locref(*gp_fixargs((in_route_features, update_all_events, event_layers), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('GenerateEvents_locref', None)
def GenerateEvents(in_event_layer=None):
    """GenerateEvents_locref(in_event_layer)

        Re-creates shapes for event features registered with an LRS Network.
        If the event has derived network route and measure fields configured,
        the tool will update those fields when executed.

     INPUTS:
      in_event_layer (Feature Layer):
          The event for which shapes will be regenerated."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.GenerateEvents_locref(*gp_fixargs((in_event_layer,), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('GenerateRoutes_locref', None)
def GenerateRoutes(in_route_features=None, record_calibration_changes=None):
    """GenerateRoutes_locref(in_route_features, {record_calibration_changes})

        Re-creates shapes and applies calibration changes for route features
        in an LRS Network.Additionally, the tool can determine calibration
        changes on routes so
        event behaviors can be processed. These calibration changes can be due
        to adding, editing, or deleting calibration points outside the
        Location Referencing tools in ArcGIS Pro.

     INPUTS:
      in_route_features (Feature Layer):
          The LRS Network for which route shapes will be regenerated and
          calibration changes will be applied.
      record_calibration_changes {Boolean}:
          * RECORD_CALIBRATION_CHANGES-Any calibration points created, modified,
          or deleted outside the ArcGIS Pipeline Referencing tools will be
          applied to the routes in the network and event behaviors will be
          applied the next time Apply Event Behaviors is run.

          * NO_RECORD_CALIBRATION_CHANGES-Calibration changes will be applied to
          the routes in the LRS Network, but no event behaviors will be applied.
          This is the default."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.GenerateRoutes_locref(*gp_fixargs((in_route_features, record_calibration_changes), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('OverlayEvents_locref', None)
def OverlayEvents(in_route_features=None, event_layers=None, output_dataset=None, include_geometry=None, network_fields=None):
    """OverlayEvents_locref(in_route_features, event_layers;event_layers..., output_dataset, {include_geometry}, {network_fields;network_fields...})

        Overlays one or more linear event feature layers against a target
        network and outputs a feature class or table that represents the
        dynamic segmentation of the inputs.

     INPUTS:
      in_route_features (Feature Layer):
          The target network against which the event layers will be dynamically
          segmented.
      event_layers (Feature Layer):
          The event layers to be dynamically segmented together against a
          target network.
      include_geometry {Boolean}:
          Determines whether the output_dataset will include a geometry column.

          * EXCLUDE_GEOMETRY-Will not include the event geometry in the
          output_dataset. Event records will be stored as a table. This is the
          default.

          * INCLUDE_GEOMETRY-Will include the event geometry in the
          output_dataset. Event records will be stored as a feature class.
      network_fields {Field}:
          Fields from the network layer that will be included in the output.

     OUTPUTS:
      output_dataset (Table):
          The table or feature class to be created, containing the output event
          records."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.OverlayEvents_locref(*gp_fixargs((in_route_features, event_layers, output_dataset, include_geometry, network_fields), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('TranslateEventMeasures_locref', None)
def TranslateEventMeasures(in_source_event=None, in_target_route_features=None, out_target_event=None, in_concurrent_route_matching=None):
    """TranslateEventMeasures_locref(in_source_event, in_target_route_features, out_target_event, {in_concurrent_route_matching})

        Translates the measures (m-values) of events from one LRM (Linear
        Referencing Method) to another. The output is a new feature class.This
        tool brings together event data provided by various business
        units that may use different LRMs. These LRMs may have different route
        IDs and m-values that require translation between LRMs.

     INPUTS:
      in_source_event (Feature Layer):
          The source event layer to be translated.
      in_target_route_features (Feature Layer):
          The route network to which the source events will be translated.
      in_concurrent_route_matching {String}:
          The method to determine which route to translate the event onto when
          concurrent routes exist in the target network. This selected method is
          only applied when the location of the event translation in the target
          network has concurrent routes.

          * ANY-The source event is translated against the first of the
          concurrent routes at the source event location in the target network.

          * ROUTE_ID-The RouteID of the source event is compared to the RouteIDs
          of concurrent routes at the source event location in the target
          network. The source event will translate based on RouteIDs matching in
          the source event and target network. The RouteIDs of the source event
          and target network must be an exact match for this method to correctly
          translate the event.

          * ALL-The source event is translated against all the concurrent routes
          at that location in the target network.

     OUTPUTS:
      out_target_event (Feature Class):
          The translated event to be created."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.TranslateEventMeasures_locref(*gp_fixargs((in_source_event, in_target_route_features, out_target_event, in_concurrent_route_matching), True)))
        return retval
    except Exception as e:
        raise e


# Configuration toolset
@gptooldoc('RemoveLRSEntity_locref', None)
def RemoveLRSEntity(in_workspace=None, lrs_entity_type=None, lrs_entity_name=None):
    """RemoveLRSEntity_locref(in_workspace, lrs_entity_type, lrs_entity_name)

        Removes a linear referencing system (LRS) entity from an input
        geodatabase workspace.

     INPUTS:
      in_workspace (Workspace):
          The input geodatabase workspace that contains the LRS entity that
          will be removed.
      lrs_entity_type (String):
          The type of LRS entity that will be removed from the input
          geodatabase workspace.

          * LRS-Removes an LRS as well as all dependent LRS Networks, LRS
          events, and LRS intersections registered to those LRS Networks.

          * NETWORK-Removes an LRS Network as well as all LRS events and LRS
          intersections registered to that LRS Network.

          * EVENT-Removes an LRS event.

          * INTERSECTION-Removes an LRS intersection.
      lrs_entity_name (String):
          The name of the LRS entity that will be removed from the input
          geodatabase workspace. The underlying feature classes and tables of
          the LRS entity will not be deleted."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.RemoveLRSEntity_locref(*gp_fixargs((in_workspace, lrs_entity_type, lrs_entity_name), True)))
        return retval
    except Exception as e:
        raise e


# Configuration\LRS toolset
@gptooldoc('CreateLRS_locref', None)
def CreateLRS(in_workspace=None, lrs_name=None, centerline_feature_class_name=None, calibration_point_feature_class_name=None, redline_feature_class_name=None, centerline_sequence_table_name=None, spatial_reference=None, xy_tolerance=None, z_tolerance=None, xy_resolution=None, z_resolution=None):
    """CreateLRS_locref(in_workspace, lrs_name, centerline_feature_class_name, calibration_point_feature_class_name, redline_feature_class_name, centerline_sequence_table_name, spatial_reference, {xy_tolerance}, {z_tolerance}, {xy_resolution}, {z_resolution})

        Creates a linear referencing system (LRS) and minimum schema items in
        a specified workspace.

     INPUTS:
      in_workspace (Workspace / Feature Dataset):
          The file or multipurpose geodatabase where the LRS and minimum schema
          will be created.
      lrs_name (String):
          The name of the output LRS.
      centerline_feature_class_name (String):
          Name of the output centerline feature class.
      calibration_point_feature_class_name (String):
          Name of the output calibration point feature class.
      redline_feature_class_name (String):
          Name of the output redline feature class.
      centerline_sequence_table_name (String):
          Name of the output centerline sequence table.
      spatial_reference (Spatial Reference):
          The spatial reference for the output feature classes. When executing
          from a Python script, you can use the Well Known ID (WKID) for the
          spatial reference.
      xy_tolerance {Linear Unit}:
          The xy-tolerance of the output feature classes.
      z_tolerance {Linear Unit}:
          The z-tolerance of the output feature classes.
      xy_resolution {Linear Unit}:
          The xy-resolution of the output feature classes.
      z_resolution {Linear Unit}:
          The z-resolution of the output feature classes."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateLRS_locref(*gp_fixargs((in_workspace, lrs_name, centerline_feature_class_name, calibration_point_feature_class_name, redline_feature_class_name, centerline_sequence_table_name, spatial_reference, xy_tolerance, z_tolerance, xy_resolution, z_resolution), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('CreateLRSFromExistingDataset_locref', None)
def CreateLRSFromExistingDataset(lrs_name=None, centerline_feature_class=None, centerline_centerline_id_field=None, centerline_sequence_table=None, centerline_sequence_centerline_id_field=None, centerline_sequence_route_id_field=None, centerline_sequence_from_date_field=None, centerline_sequence_to_date_field=None, centerline_sequence_network_id_field=None, calibration_point_feature_class=None, calibration_point_measure_field=None, calibration_point_from_date_field=None, calibration_point_to_date_field=None, calibration_point_route_id_field=None, calibration_point_network_id_field=None, redline_feature_class=None, redline_from_measure_field=None, redline_to_measure_field=None, redline_route_id_field=None, redline_route_name_field=None, redline_effective_date_field=None, redline_activity_type_field=None, redline_network_id_field=None):
    """CreateLRSFromExistingDataset_locref(lrs_name, centerline_feature_class, centerline_centerline_id_field, centerline_sequence_table, centerline_sequence_centerline_id_field, centerline_sequence_route_id_field, centerline_sequence_from_date_field, centerline_sequence_to_date_field, centerline_sequence_network_id_field, calibration_point_feature_class, calibration_point_measure_field, calibration_point_from_date_field, calibration_point_to_date_field, calibration_point_route_id_field, calibration_point_network_id_field, redline_feature_class, redline_from_measure_field, redline_to_measure_field, redline_route_id_field, redline_route_name_field, redline_effective_date_field, redline_activity_type_field, redline_network_id_field)

        Creates a linear referencing system (LRS) in the specified workspace
        using existing datasets.

     INPUTS:
      lrs_name (String):
          The name of the LRS being created. The name for the LRS should not
          already exist in the geodatabase.
      centerline_feature_class (Feature Layer):
          The feature class to be used as the centerline in the LRS.
      centerline_centerline_id_field (Field):
          The field containing the centerline ID. GUIDs are supported. The field
          type should match the centerlineID field type in the Centerline
          Sequence table.
      centerline_sequence_table (Table View):
          The table to be used as the centerline sequence in the LRS.
      centerline_sequence_centerline_id_field (Field):
          The field containing the centerline ID. GUIDs are supported. The field
          type should match the centerlineID field type and length in the
          Centerline feature class.
      centerline_sequence_route_id_field (Field):
          The field containing the route ID. GUID and text field types are
          supported. The field type should match the routeID field type and
          length in the Calibration Point and Redline feature classes.
      centerline_sequence_from_date_field (Field):
          A date field containing the from date.
      centerline_sequence_to_date_field (Field):
          A date field containing the to date.
      centerline_sequence_network_id_field (Field):
          The field containing the network ID. Short Integer field type is
          supported.
      calibration_point_feature_class (Feature Layer):
          The feature class to be used as the calibration point in the LRS.
      calibration_point_measure_field (Field):
          The field containing the measure. Double field type is supported.
      calibration_point_from_date_field (Field):
          A date field containing the from date.
      calibration_point_to_date_field (Field):
          A date field containing the to date.
      calibration_point_route_id_field (Field):
          The field containing the route ID. GUID and text field types are
          supported. The field type should match the routeID field type and
          length in the Centerline Sequence table and Redline feature class.
      calibration_point_network_id_field (Field):
          The field containing the network ID. Short Integer field type is
          supported.
      redline_feature_class (Feature Layer):
          The feature class being used as the redline in the LRS.
      redline_from_measure_field (Field):
          The field containing the from measure. Double field type is supported.
      redline_to_measure_field (Field):
          The field containing the to measure. Double field type is supported.
      redline_route_id_field (Field):
          The field containing the route ID. GUID and text field types are
          supported. The field type should match the routeID field type and
          length in the Calibration Point feature class and Centerline Sequence
          table.
      redline_route_name_field (Field):
          A text field containing the route name.
      redline_effective_date_field (Field):
          A date field containing the effective date.
      redline_activity_type_field (Field):
          The field containing the activity type. Short Integer field type is
          supported.
      redline_network_id_field (Field):
          The field containing the network ID. Short Integer field type is
          supported."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateLRSFromExistingDataset_locref(*gp_fixargs((lrs_name, centerline_feature_class, centerline_centerline_id_field, centerline_sequence_table, centerline_sequence_centerline_id_field, centerline_sequence_route_id_field, centerline_sequence_from_date_field, centerline_sequence_to_date_field, centerline_sequence_network_id_field, calibration_point_feature_class, calibration_point_measure_field, calibration_point_from_date_field, calibration_point_to_date_field, calibration_point_route_id_field, calibration_point_network_id_field, redline_feature_class, redline_from_measure_field, redline_to_measure_field, redline_route_id_field, redline_route_name_field, redline_effective_date_field, redline_activity_type_field, redline_network_id_field), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('ModifyLRS_locref', None)
def ModifyLRS(in_workspace=None, current_lrs_name=None, new_lrs_name=None, centerline_feature_class=None, centerline_centerline_id_field=None, centerline_sequence_table=None, centerline_sequence_centerline_id_field=None, centerline_sequence_route_id_field=None, centerline_sequence_from_date_field=None, centerline_sequence_to_date_field=None, centerline_sequence_network_id_field=None, calibration_point_feature_class=None, calibration_point_measure_field=None, calibration_point_from_date_field=None, calibration_point_to_date_field=None, calibration_point_route_id_field=None, calibration_point_network_id_field=None, redline_feature_class=None, redline_from_measure_field=None, redline_to_measure_field=None, redline_route_id_field=None, redline_route_name_field=None, redline_effective_date_field=None, redline_activity_type_field=None, redline_network_id_field=None):
    """ModifyLRS_locref(in_workspace, current_lrs_name, {new_lrs_name}, {centerline_feature_class}, {centerline_centerline_id_field}, {centerline_sequence_table}, {centerline_sequence_centerline_id_field}, {centerline_sequence_route_id_field}, {centerline_sequence_from_date_field}, {centerline_sequence_to_date_field}, {centerline_sequence_network_id_field}, {calibration_point_feature_class}, {calibration_point_measure_field}, {calibration_point_from_date_field}, {calibration_point_to_date_field}, {calibration_point_route_id_field}, {calibration_point_network_id_field}, {redline_feature_class}, {redline_from_measure_field}, {redline_to_measure_field}, {redline_route_id_field}, {redline_route_name_field}, {redline_effective_date_field}, {redline_activity_type_field}, {redline_network_id_field})

        Modifies an existing linear referencing system (LRS) in the specified
        workspace.

     INPUTS:
      in_workspace (Workspace):
          The LRS workspace.
      current_lrs_name (String):
          Name of the current LRS.
      new_lrs_name {String}:
          New name for the current LRS.
      centerline_feature_class {Feature Layer}:
          An existing centerline feature class for the minimum schema.
      centerline_centerline_id_field {Field}:
          Name of the centerline ID field from the existing centerline feature
          class that was chosen.
      centerline_sequence_table {Table View}:
          An existing centerline sequence table for the minimum schema.
      centerline_sequence_centerline_id_field {Field}:
          Name of the centerline ID field from the existing centerline sequence
          table that was chosen.
      centerline_sequence_route_id_field {Field}:
          Name of the route ID field from the existing centerline sequence
          table that was chosen.
      centerline_sequence_from_date_field {Field}:
          Name of the from date field from the existing centerline sequence
          table that was chosen.
      centerline_sequence_to_date_field {Field}:
          Name of the to date field from the existing centerline sequence table
          that was chosen.
      centerline_sequence_network_id_field {Field}:
          Name of the network ID field from the existing centerline sequence
          table that was chosen.
      calibration_point_feature_class {Feature Layer}:
          An existing calibration point feature class for the minimum schema.
      calibration_point_measure_field {Field}:
          Name of the measure field from the existing calibration point feature
          class that was chosen.
      calibration_point_from_date_field {Field}:
          Name of the from date field from the existing calibration point
          feature class that was chosen.
      calibration_point_to_date_field {Field}:
          Name of the to date field from the existing calibration point feature
          class that was chosen.
      calibration_point_route_id_field {Field}:
          Name of the route ID field from the existing calibration point
          feature class that was chosen.
      calibration_point_network_id_field {Field}:
          Name of the network ID field from the existing calibration point
          feature class that was chosen.
      redline_feature_class {Feature Layer}:
          An existing redline feature class for the minimum schema.
      redline_from_measure_field {Field}:
          Name of the from measure field from the existing redline feature
          class that was chosen.
      redline_to_measure_field {Field}:
          Name of the to measure field from the existing redline feature class
          that was chosen.
      redline_route_id_field {Field}:
          Name of the route ID field from the existing redline feature class
          that was chosen.
      redline_route_name_field {Field}:
          Name of the route name field from the existing chosen redline feature
          class that was chosen.
      redline_effective_date_field {Field}:
          Name of the effective date field from the existing redline feature
          class that was chosen.
      redline_activity_type_field {Field}:
          Name of the activity type field from the existing redline feature
          class that was chosen.
      redline_network_id_field {Field}:
          Name of the network ID field from the existing redline feature class
          that was chosen."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ModifyLRS_locref(*gp_fixargs((in_workspace, current_lrs_name, new_lrs_name, centerline_feature_class, centerline_centerline_id_field, centerline_sequence_table, centerline_sequence_centerline_id_field, centerline_sequence_route_id_field, centerline_sequence_from_date_field, centerline_sequence_to_date_field, centerline_sequence_network_id_field, calibration_point_feature_class, calibration_point_measure_field, calibration_point_from_date_field, calibration_point_to_date_field, calibration_point_route_id_field, calibration_point_network_id_field, redline_feature_class, redline_from_measure_field, redline_to_measure_field, redline_route_id_field, redline_route_name_field, redline_effective_date_field, redline_activity_type_field, redline_network_id_field), True)))
        return retval
    except Exception as e:
        raise e


# Configuration\LRS Event toolset
@gptooldoc('CreateLRSEvent_locref', None)
def CreateLRSEvent(parent_network=None, event_name=None, geometry_type=None, event_id_field=None, route_id_field=None, from_date_field=None, to_date_field=None, loc_error_field=None, measure_field=None, to_measure_field=None, event_spans_routes=None, to_route_id_field=None, store_route_name=None, route_name_field=None, to_route_name_field=None):
    """CreateLRSEvent_locref(parent_network, event_name, geometry_type, event_id_field, route_id_field, from_date_field, to_date_field, loc_error_field, measure_field, {to_measure_field}, {event_spans_routes}, {to_route_id_field}, {store_route_name}, {route_name_field}, {to_route_name_field})

        Creates line or point events for an existing LRS Network.

     INPUTS:
      parent_network (Feature Layer):
          The network to which the event is registered.
      event_name (String):
          The event to be registered.
      geometry_type (String):
          The geometry type of the output event.

          * POINT-The geometry type of the event is Point. This is the default.

          * LINE-The geometry type of the event is Polyline.
      event_id_field (String):
          The event ID field available in the event feature class.
      route_id_field (String):
          Name of the route ID field if it is a point event that does not span
          routes, or from route ID field if the event_spans_routes parameter is
          set to SPANS_ROUTES.
      from_date_field (String):
          The from date field available in the event feature class.
      to_date_field (String):
          The to date field available in the event feature class.
      loc_error_field (String):
          The location error field available in the event feature class.
      measure_field (String):
          Name of the measure field if it is a point event or from measure
          field if it is a line event.
      to_measure_field {String}:
          Name of the to measure field. Required only for Line events.
      event_spans_routes {Boolean}:
          Determines if the event records spans routes.

          * SPANS_ROUTES-The event records span routes.

          * NO_SPANS_ROUTES-The event records do not span routes. This is the
          default.
      to_route_id_field {String}:
          Name of the to route ID field. Required only if the geometry_type
          parameter is set to LINE and the event_span_routes parameter is set to
          SPANS_ROUTES.
      store_route_name {Boolean}:
          Determines if the route name should be stored with the event records.

          * STORE_ROUTE_NAME-Stores the route name with the event records.

          * NO_STORE_ROUTE_NAME-Does not store the route name with the event
          records. This is the default.
      route_name_field {String}:
          The route name field if it is a point event that does not span
          routes, or the from route name field if it is a line event that spans
          routes. Required if STORE_ROUTE_NAME is set.
      to_route_name_field {String}:
          The to route name field for line events that span routes. Required if
          STORE_ROUTE_NAME is set."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateLRSEvent_locref(*gp_fixargs((parent_network, event_name, geometry_type, event_id_field, route_id_field, from_date_field, to_date_field, loc_error_field, measure_field, to_measure_field, event_spans_routes, to_route_id_field, store_route_name, route_name_field, to_route_name_field), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('CreateLRSEventFromExistingDataset_locref', None)
def CreateLRSEventFromExistingDataset(parent_network=None, in_feature_class=None, event_id_field=None, route_id_field=None, from_date_field=None, to_date_field=None, loc_error_field=None, measure_field=None, to_measure_field=None, event_spans_routes=None, to_route_id_field=None, store_route_name=None, route_name_field=None, to_route_name_field=None):
    """CreateLRSEventFromExistingDataset_locref(parent_network, in_feature_class, event_id_field, route_id_field, from_date_field, to_date_field, loc_error_field, measure_field, {to_measure_field}, {event_spans_routes}, {to_route_id_field}, {store_route_name}, {route_name_field}, {to_route_name_field})

        Registers an existing feature class as an event.

     INPUTS:
      parent_network (Feature Layer):
          The network to which the event will be registered.
      in_feature_class (Feature Layer):
          The event to be registered.
      event_id_field (Field):
          The event ID field in the event feature class.
      route_id_field (Field):
          The route ID field if the feature class is a point event or from
          route ID field if the feature class is a line event and
          event_spans_routes is set to SPANS_ROUTES.
      from_date_field (Field):
          The from date field in the event feature class.
      to_date_field (Field):
          The to date field in the event feature class.
      loc_error_field (Field):
          The location error field in the event feature class.
      measure_field (Field):
          The measure field if the feature class is a point event, or from
          measure field if the feature class is a line event.
      to_measure_field {Field}:
          The to measure field in the event feature class. Required for line
          events.
      event_spans_routes {Boolean}:
          Determines if the event records will span routes.

          * NO_SPANS_ROUTES-The event records do not span routes. This is the
          default.

          * SPANS_ROUTES-The event records may span routes.
      to_route_id_field {Field}:
          The to route ID field for events that span routes. Required if
          in_feature class geometry type is polyline and event_span_routes is
          set to SPANS_ROUTES.
      store_route_name {Boolean}:
          Determines if the route name should be stored with the event records.

          * NO_STORE_ROUTE_NAME-Does not store the route name with the event
          records. This is the default.

          * STORE_ROUTE_NAME-Stores the route name with the event records.
      route_name_field {Field}:
          The route name field if the feature class is a point event that
          doesn't span routes, or the from route name field if the feature class
          is a line event that spans routes. Required if STORE_ROUTE_NAME is
          set.
      to_route_name_field {Field}:
          The to route name field for line events that span routes. Required if
          STORE_ROUTE_NAME is set."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateLRSEventFromExistingDataset_locref(*gp_fixargs((parent_network, in_feature_class, event_id_field, route_id_field, from_date_field, to_date_field, loc_error_field, measure_field, to_measure_field, event_spans_routes, to_route_id_field, store_route_name, route_name_field, to_route_name_field), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('DisableReferentFields_locref', None)
def DisableReferentFields(in_feature_class=None):
    """DisableReferentFields_locref(in_feature_class)

        Disables referent fields for an existing linear referencing system
        (LRS) event feature class or feature layer. It does not delete the
        referent columns; it removes the referent column information from the
        Lrs_Metadata table.

     INPUTS:
      in_feature_class (Feature Layer):
          The input feature class or feature layer for the LRS event."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.DisableReferentFields_locref(*gp_fixargs((in_feature_class,), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('EnableReferentFields_locref', None)
def EnableReferentFields(in_feature_class=None, from_referent_method_field=None, from_referent_location_field=None, from_referent_offset_field=None, to_referent_method_field=None, to_referent_location_field=None, to_referent_offset_field=None, offset_units=None):
    """EnableReferentFields_locref(in_feature_class, {from_referent_method_field}, {from_referent_location_field}, {from_referent_offset_field}, {to_referent_method_field}, {to_referent_location_field}, {to_referent_offset_field}, {offset_units})

        Enables or modifies the referent fields so that you can manage
        referent information for the registered LRS event. It also support
        modifying the existing referent fields.For more information, see
        Storing referent and offset information for
        event location.

     INPUTS:
      in_feature_class (Feature Layer):
          The feature class for the LRS event.
      from_referent_method_field {Field}:
          The from referent method field.
      from_referent_location_field {Field}:
          The from referent location field.
      from_referent_offset_field {Field}:
          The from referent offset field.
      to_referent_method_field {Field}:
          The to referent method field.
      to_referent_location_field {Field}:
          The to referent location field.
      to_referent_offset_field {Field}:
          The to referent offset field.
      offset_units {String}:
          The offset units. Feet is the default.

          * CENTIMETERS-Centimeters is the offset unit.

          * DECIMETERS-Decimeters is the offset unit.

          * FEET-Feet is the offset unit.

          * INCHES-Inches is the offset unit.

          * KILOMETERS-Kilometers is the offset unit.

          * METERS-Meters is the offset unit.

          * MILES-Miles is the offset unit.

          * MILLIMETERS-Millimeters is the offset unit.

          * YARDS-Yards is the offset unit."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.EnableReferentFields_locref(*gp_fixargs((in_feature_class, from_referent_method_field, from_referent_location_field, from_referent_offset_field, to_referent_method_field, to_referent_location_field, to_referent_offset_field, offset_units), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('ModifyEventBehaviorRules_locref', None)
def ModifyEventBehaviorRules(in_feature_class=None, calibrate_rule=None, retire_rule=None, extend_rule=None, reassign_rule=None, realign_rule=None):
    """ModifyEventBehaviorRules_locref(in_feature_class, {calibrate_rule}, {retire_rule}, {extend_rule}, {reassign_rule}, {realign_rule})

        Modifies event behavior rules for the registered event layer or
        feature class.

     INPUTS:
      in_feature_class (Feature Layer):
          The event feature class.
      calibrate_rule {String}:
          The event behavior rule defined for the calibrate activity.

          * STAY_PUT-Preserves the geographic location of the event; measures
          may change. This is the default.

          * RETIRE-Preserves both measure and geographic location; event is
          retired.

          * MOVE-Preserves the measures of the event; geographic location may
          change.
      retire_rule {String}:
          The event behavior rule defined for the retire activity.

          * STAY_PUT-Preserves the geographic location of the event; measures
          may change. This is the default.

          * RETIRE-Preserves both measure and geographic location; event is
          retired.

          * MOVE-Preserves the measures of the event; geographic location may
          change.
      extend_rule {String}:
          The event behavior rule defined for the extend activity.

          * STAY_PUT-Preserves the geographic location of the event; measures
          may change. This is the default.

          * RETIRE-Preserves both measure and geographic location; event is
          retired.

          * MOVE-Preserves the measures of the event; geographic location may
          change.
      reassign_rule {String}:
          The event behavior rule defined for the reassign activity.

          * STAY_PUT-Preserves the geographic location of the event; measures
          may change. This is the default.

          * RETIRE-Preserves both measure and geographic location; event is
          retired.

          * MOVE-Preserves the measures of the event; geographic location may
          change.

          * SNAP-Preserves the location of an event by snapping the event to a
          concurrent route; measures may change.
      realign_rule {String}:
          The event behavior rule defined for the realign activity.

          * STAY_PUT-Preserves the geographic location of the event; measures
          may change. This is the default.

          * RETIRE-Preserves both measure and geographic location; event is
          retired.

          * MOVE-Preserves the measures of the event; geographic location may
          change."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ModifyEventBehaviorRules_locref(*gp_fixargs((in_feature_class, calibrate_rule, retire_rule, extend_rule, reassign_rule, realign_rule), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('ModifyLRSEvent_locref', None)
def ModifyLRSEvent(in_feature_class=None, event_id_field=None, route_id_field=None, from_date_field=None, to_date_field=None, loc_error_field=None, measure_field=None, to_measure_field=None, event_spans_routes=None, to_route_id_field=None, store_route_name=None, route_name_field=None, to_route_name_field=None):
    """ModifyLRSEvent_locref(in_feature_class, {event_id_field}, {route_id_field}, {from_date_field}, {to_date_field}, {loc_error_field}, {measure_field}, {to_measure_field}, {event_spans_routes}, {to_route_id_field}, {store_route_name}, {route_name_field}, {to_route_name_field})

        Modifies properties of a linear referencing system (LRS) event.

     INPUTS:
      in_feature_class (Feature Layer):
          The input feature class or feature layer for the event.
      event_id_field {Field}:
          Name of the event ID field.
      route_id_field {Field}:
          Name of the route ID field.
      from_date_field {Field}:
          Name of the from date field.
      to_date_field {Field}:
          Name of the to date field.
      loc_error_field {Field}:
          Name of the location error field.
      measure_field {Field}:
          Name of the measure field if it is a point event or from measure
          field if it is a line event.
      to_measure_field {Field}:
          Name of the to measure field. Required only for line events.
      event_spans_routes {String}:
          Determines if the event records will span routes.

          * AS_IS-No change to the property. This is the default.

          * NO_SPANS_ROUTES-The event records do not span routes. Applicable
          only for line events.

          * SPANS_ROUTES-The event records may span routes. Applicable only for
          line events.
      to_route_id_field {Field}:
          Name of the to route ID field. Required only if it is a line event
          and it spans routes.
      store_route_name {String}:
          Determines if the event records will store the route name.

          * AS_IS-No change to the property. This is the default.

          * STORE_ROUTE_NAME-Stores the route name with the event records.

          * NO_STORE_ROUTE_NAME-Does not store the route name with the event
          records.
      route_name_field {Field}:
          The route name field if it is a point event that does not span
          routes, or the from route name field if it is a line event that spans
          routes. Required if STORE_ROUTE_NAME is set.
      to_route_name_field {Field}:
          Name of the to route name field. Required if it is a line event, and
          store_route_name and SPANS_ROUTES are set."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ModifyLRSEvent_locref(*gp_fixargs((in_feature_class, event_id_field, route_id_field, from_date_field, to_date_field, loc_error_field, measure_field, to_measure_field, event_spans_routes, to_route_id_field, store_route_name, route_name_field, to_route_name_field), True)))
        return retval
    except Exception as e:
        raise e


# Configuration\LRS Network toolset
@gptooldoc('CreateLRSNetwork_locref', None)
def CreateLRSNetwork(in_path=None, lrs_name=None, network_name=None, route_id_field=None, route_name_field=None, from_date_field=None, to_date_field=None, derive_from_line_network=None, line_network_name=None, include_fields_to_support_lines=None, line_id_field=None, line_name_field=None, line_order_field=None, measure_unit=None):
    """CreateLRSNetwork_locref(in_path, lrs_name, network_name, route_id_field, route_name_field, from_date_field, to_date_field, {derive_from_line_network}, {line_network_name}, {include_fields_to_support_lines}, {line_id_field}, {line_name_field}, {line_order_field}, {measure_unit})

        Creates an LRS Network within an ArcGIS Pipeline Referencing linear
        referencing system (LRS).

     INPUTS:
      in_path (Workspace / Feature Dataset):
          The input workspace that will contain the new LRS Network. This
          workspace must be a geodatabase that contains a Pipeline Referencing
          LRS. In addition to the top level of a geodatabase, a feature dataset
          is also supported as a valid path.
      lrs_name (String):
          The LRS to which the new LRS Network will be registered. The LRS must
          reside in the same geodatabase as the in_path.
      network_name (String):
          The name of the new LRS Network that will be created, as well as the
          name of the new feature class that will be created and registered with
          the LRS Network. The LRS name must be 26 or fewer characters and not
          contain any special characters other than space, hyphen, or
          underscore.
      route_id_field (String):
          The field in the output feature class that will be mapped as the LRS
          Network route ID. The field type is derived from the RouteId field of
          the Centerline_Sequence table and will either be a string or GUID
          field type.
      route_name_field (String):
          A string field in the output feature class that will be mapped as the
          LRS Network route name.
      from_date_field (String):
          A date field in the output feature class that will be mapped as the
          LRS Network from date.
      to_date_field (String):
          A date field in the output feature class that will be mapped as the
          LRS Network to date.
      derive_from_line_network {Boolean}:
          Determines if the network will be configured as an LRS Derived
          Network.

          * DERIVE-The output of this tool will be an LRS Derived Network and a
          feature class to support the LRS Derived Network. The
          line_network_name parameter must also be provided.

          * DO_NOT_DERIVE-The output of this tool will not be an LRS Derived
          Network. This is the default.
      line_network_name {String}:
          The name of the LRS Line Network to which the output LRS Derived
          Network will be registered. The input LRS Line Network must reside in
          the same geodatabase workspace as the line_network_name. This
          parameter is only used if the derive_from_line_network parameter is
          set to DERIVE.
      include_fields_to_support_lines {Boolean}:
          Determines whether to add fields that support lines.

          * INCLUDE-The output of this tool will be an LRS Line Network, and
          the output feature class will include fields needed to support lines.
          The line_id_field, line_name_field, and line_order_field parameters
          must also be provided.

          * DO_NOT_INCLUDE-The output of this tool will not be an LRS Line
          Network. This is the default.
      line_id_field {String}:
          The field in the output feature class that will be mapped as the LRS
          Network line ID. This parameter is only used if the
          include_fields_to_support_lines parameter is set to INCLUDE. The field
          type is derived from the RouteId field of the Centerline_Sequence
          table and will either be a string of exactly 38 characters or a GUID
          field type.
      line_name_field {String}:
          A string field in the output feature class that will be mapped as the
          LRS Network line name. This parameter is only used if the
          include_fields_to_support_lines parameter is set to INCLUDE.
      line_order_field {String}:
          The field in the output feature class that will be mapped as the LRS
          Network line order. This parameter is only used if the
          include_fields_to_support_lines parameter is set to INCLUDE. This will
          be a long integer field type.
      measure_unit {String}:
          The unit of measure (m units) that the LRS Network will use. The
          following inputs are valid:

          * MILES-This is the default.

          * INCHES

          * FEET

          * YARDS

          * NAUTICAL_MILES

          * MILLIMETERS

          * CENTIMETERS

          * METERS

          * KILOMETERS

          * DECIMETERS"""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateLRSNetwork_locref(*gp_fixargs((in_path, lrs_name, network_name, route_id_field, route_name_field, from_date_field, to_date_field, derive_from_line_network, line_network_name, include_fields_to_support_lines, line_id_field, line_name_field, line_order_field, measure_unit), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('CreateLRSNetworkFromExistingDataset_locref', None)
def CreateLRSNetworkFromExistingDataset(in_feature_class=None, lrs_name=None, route_id_field=None, route_name_field=None, from_date_field=None, to_date_field=None, derive_from_line_network=None, line_network_name=None, include_fields_to_support_lines=None, line_id_field=None, line_name_field=None, line_order_field=None):
    """CreateLRSNetworkFromExistingDataset_locref(in_feature_class, lrs_name, route_id_field, route_name_field, from_date_field, to_date_field, {derive_from_line_network}, {line_network_name}, {include_fields_to_support_lines}, {line_id_field}, {line_name_field}, {line_order_field})

        Creates an LRS Network using an existing polyline feature class.

     INPUTS:
      in_feature_class (Feature Layer):
          The input feature class that will be registered as the LRS Network.
          The feature class's name must be 26 or fewer characters. The feature
          class must reside within a geodatabase that contains a Pipeline
          Referencing LRS. The name of this feature class will be used as the
          name of the LRS Network.
      lrs_name (String):
          The LRS name to which the new LRS Network will be registered. The LRS
          must reside in the same geodatabase as the in_feature_class.
      route_id_field (Field):
          The field in the in_feature_class that will be mapped as the LRS
          Network route ID. This must be a string or GUID field type.
      route_name_field (Field):
          A string field in the in_feature_class that will be mapped as the LRS
          Network route name.
      from_date_field (Field):
          A date field in the in_feature_class that will be mapped as the LRS
          Network from date.
      to_date_field (Field):
          A date field in the in_feature_class that will be mapped as the LRS
          Network to date.
      derive_from_line_network {Boolean}:
          Determines if the network will be configured as an LRS Derived
          Network.

          * DERIVE-The output of this tool will be an LRS Derived Network. The
          line_network_name parameter must also be provided.

          * DO_NOT_DERIVE-The output of this tool will not be an LRS Derived
          Network. This is the default.
      line_network_name {String}:
          The name of the LRS Line Network to which the output LRS Derived
          Network will be registered. The input LRS Line Network must reside in
          the same geodatabase workspace as the in_feature_class. This parameter
          is only used if the derive_from_line_network parameter is set to
          DERIVE.
      include_fields_to_support_lines {Boolean}:
          Determines if the network will be configured to support lines.

          *  INCLUDE-The output of this tool will be an LRS Line Network. The
          line_id_field, line_name_field, and line_order_field parameters must
          also be provided.

          * DO_NOT_INCLUDE-The output of this tool will not be an LRS Line
          Network. This is the default.
      line_id_field {Field}:
          The field in the in_feature_class that will be mapped as the LRS
          Network line ID. This parameter is only used if the
          include_fields_to_support_lines parameter is set to INCLUDE. This must
          be a string or GUID field type and must match the field type and
          length of the route ID field of the Centerline_Sequence table.
      line_name_field {Field}:
          A string field in the in_feature_class that will be mapped as the LRS
          Network line name. This parameter is only used if the
          include_fields_to_support_lines parameter is set to INCLUDE.
      line_order_field {Field}:
          An integer field in the in_feature_class that will be mapped as the
          LRS Network line order. This parameter is only used if the
          include_fields_to_support_lines parameter is set to INCLUDE."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateLRSNetworkFromExistingDataset_locref(*gp_fixargs((in_feature_class, lrs_name, route_id_field, route_name_field, from_date_field, to_date_field, derive_from_line_network, line_network_name, include_fields_to_support_lines, line_id_field, line_name_field, line_order_field), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('ModifyLRSNetwork_locref', None)
def ModifyLRSNetwork(in_feature_class=None, route_id_field=None, route_name_field=None, from_date_field=None, to_date_field=None, derive_from_line_network=None, line_network_name=None, include_fields_to_support_lines=None, line_id_field=None, line_name_field=None, line_order_field=None):
    """ModifyLRSNetwork_locref(in_feature_class, {route_id_field}, {route_name_field}, {from_date_field}, {to_date_field}, {derive_from_line_network}, {line_network_name}, {include_fields_to_support_lines}, {line_id_field}, {line_name_field}, {line_order_field})

        Modifies an LRS Network within an ArcGIS Pipeline Referencing linear
        referencing system (LRS).

     INPUTS:
      in_feature_class (Feature Layer):
          The input LRS Network feature class to be modified.
      route_id_field {Field}:
          The field in the input feature class that will be mapped as the LRS
          Network route ID. The field type must match the RouteId field type of
          the Centerline_Sequence table and must either be a string or GUID
          field type.
      route_name_field {Field}:
          A string field in the input feature class that will be mapped as the
          LRS Network route name.
      from_date_field {Field}:
          A date field in the input feature class that will be mapped as the
          LRS Network from date.
      to_date_field {Field}:
          A date field in the input feature class that will be mapped as the
          LRS Network to date.
      derive_from_line_network {String}:
          Determines if the current LRS network will be configured as an LRS
          Derived Network.

          * AS_IS-The current LRS Network derived property will not be changed.
          This is the default.

          * DERIVE-The input LRS Derived Network will be modified to become an
          LRS Derived Network. The line network name parameter must also be
          provided to specify which LRS Network to derive from.

          * DO_NOT_DERIVE-The input LRS Derived Network will be modified to no
          longer be an LRS Derived Network.
      line_network_name {String}:
          The name of the LRS Line Network to which the input LRS Derived
          Network will be registered. The input LRS Line Network must reside in
          the same geodatabase workspace and LRS as the LRS network feature
          class. This parameter is only used if the derive_from_line_network
          parameter is set to DERIVE.
      include_fields_to_support_lines {String}:
          Determines if the current LRS network will be configured to support
          lines.

          * AS_IS-The current LRS Network line support property will not be
          changed. This is the default.

          * INCLUDE-The input LRS Network will be modified to add support for
          lines. The line id field, line name field, and line order field
          parameters must also be provided, and valid fields to map to these
          parameters must exist in the LRS network feature class.

          * DO_NOT_INCLUDE-The input LRS Network will be modified to remove
          support for lines.
      line_id_field {Field}:
          The field in the input feature class that will be mapped as the LRS
          Network line ID. This parameter is only used if the Include Fields to
          Support Lines parameter is set to Include. The field type must match
          the RouteId field type of the Centerline_Sequence table and must
          either be a string of exactly 38 characters or a GUID field type. The
          field in the input feature class that will be mapped as the LRS
          Network line ID. This parameter is only used if the
          include_fields_to_support_lines parameter is set to INCLUDE. The field
          type must match the RouteId field type of the Centerline_Sequence
          table and must either be a string of exactly 38 characters or a GUID
          field type.
      line_name_field {Field}:
          A string field in the input feature class that will be mapped as the
          LRS Network line name. This parameter is only used if the
          include_fields_to_support_lines parameter is set to INCLUDE.
      line_order_field {Field}:
          The field in the input feature class that will be mapped as the LRS
          Network line order. This parameter is only used if the Include Fields
          to Support Lines parameter is set to Include. This must be a long
          integer field type. The field in the input feature class that will be
          mapped as the LRS
          Network line order. This parameter is only used if the
          include_fields_to_support_lines parameter is set to INCLUDE. This must
          be a long integer field type."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ModifyLRSNetwork_locref(*gp_fixargs((in_feature_class, route_id_field, route_name_field, from_date_field, to_date_field, derive_from_line_network, line_network_name, include_fields_to_support_lines, line_id_field, line_name_field, line_order_field), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject, gptoolnotsupported