# -*- coding: utf-8 -*-
#COPYRIGHT 2017 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
r"""The Space Time Pattern Mining toolbox contains statistical tools for
analyzing data distributions and patterns in the context of both space
and time. It includes a toolset for visualizing the data stored in the
space-time netCDF cube in both 2D and 3D."""
__all__ = ['CreateSpaceTimeCube', 'CreateSpaceTimeCubeDefinedLocations',
           'EmergingHotSpotAnalysis', 'FillMissingValues',
           'LocalOutlierAnalysis', 'VisualizeSpaceTimeCube2D',
           'VisualizeSpaceTimeCube3D']
__alias__ = 'stpm'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs, gptoolnotsupported
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Tools
@gptoolnotsupported
@gptooldoc('CreateSpaceTimeCube_stpm', None)
def CreateSpaceTimeCube(in_features=None, output_cube=None, time_field=None, template_cube=None, time_step_interval=None, time_step_alignment=None, reference_time=None, distance_interval=None, summary_fields=None, aggregation_shape_type=None, defined_polygon_locations=None, location_id=None):
    """CreateSpaceTimeCube_stpm(in_features, output_cube, time_field, {template_cube}, {time_step_interval}, {time_step_alignment}, {reference_time}, {distance_interval}, {summary_fields;summary_fields...}, {aggregation_shape_type}, {defined_polygon_locations}, {location_id})

        Summarizes a set of points into a netCDF data structure by
        aggregating them into space-time bins. Within each bin, the points are
        counted and specified attributes are aggregated. For all bin
        locations, the trend for counts and summary field values are
        evaluated.

     INPUTS:
      in_features (Feature Layer):
          The input point feature class to be aggregated into space-time bins.
      time_field (Field):
          The field containing the date and time (timestamp) for each point.
          This field must be of type Date.
      template_cube {File}:
          A reference space-time cube used to define the output_cube extent of
          analysis, bin dimensions, and bin alignment.  The time_step_interval,
          distance_interval, and reference_time values are also obtained from
          the template cube.  This template cube must be a netCDF (.nc) file
          that has been created using this tool.A space-time cube created by
          aggregating into DEFINED_LOCATIONS cannot
          be used as a template_cube.
      time_step_interval {Time unit}:
          The number of seconds,  minutes, hours, days, weeks, or years that
          will represent a single time step. All points within the same
          time_step_interval and distance_interval will be aggregated. (When a
          template_cube is provided, this parameter is ignored, and the
          time_step_interval value is obtained from the template cube). Examples
          of valid entries for this parameter are 1 Weeks, 13 Days, or 1 Months.
      time_step_alignment {String}:
          Defines how aggregation will occur based on a given
          time_step_interval. If a template_cube is provided, the
          time_step_alignment associated with the template_cube overrides this
          parameter setting and the time_step_alignment of the template_cube is
          used.

          * END_TIME-Time steps align to the last time event and aggregate back
          in time.

          * START_TIME-Time steps align to the first time event and aggregate
          forward in time.

          * REFERENCE_TIME-Time steps align to a particular date/time that you
          specify. If all points in the input features have a timestamp larger
          than the reference time you provide (or it falls exactly on the start
          time of the input features), the time-step interval will begin with
          that reference time and aggregate forward in time (as occurs with a
          START_TIME alignment). If all points in the input features have a
          timestamp smaller than the reference time you provide (or it falls
          exactly on the end time of the input features), the time-step interval
          will end with that reference time and aggregate backward in time (as
          occurs with an END_TIME alignment). If the reference time you provide
          is in the middle of the time extent of your data, a time-step interval
          will be created ending with the reference time provided (as occurs
          with an END_TIME alignment); additional intervals will be created both
          before and after the reference time until the full time extent of your
          data is covered.
      reference_time {Date}:
          The date/time to use to align the time-step intervals. If you want to
          bin your data weekly from Monday to Sunday, for example, you could set
          a reference time of Sunday at midnight to ensure bins break between
          Sunday and Monday at midnight.  (When a template_cube is provided,
          this parameter is ignored and the reference_time is based on the
          template_cube.)
      distance_interval {Linear Unit}:
          The size of the bins used to aggregate the in_features. All points
          that fall within the same distance_interval and time_step_interval
          will be aggregated. When aggregating into a hexagon grid, this
          distance is used as the height to construct the hexagon polygons.
          (When a template_cube is provided, this parameter is ignored and the
          distance interval value will be based on the template_cube.)
      summary_fields {Value Table}:
          The numeric field containing attribute values used to calculate the
          specified statistic when aggregating into a space-time cube. Multiple
          statistic and field combinations can be specified. Null values are
          excluded from all statistical calculations.Available statistic types
          are:

          * SUM-Adds the total value for the specified field within each bin.

          * MEAN-Calculates the average for the specified field within each bin.

          * MIN-Finds the smallest value for all records of the specified field
          within each bin.

          * MAX-Finds the largest value for all records of the specified field
          within each bin.

          * STD-Finds the standard deviation on values in the specified field
          within each bin.

          * MEDIAN-Finds the sorted middle value of all records of the specified
          field within each bin.
          Available fill types are:

          * ZEROS-Fills empty bins with zeros.

          * SPATIAL_NEIGHBORS-Fills empty bins with the average value of spatial
          neighbors

          * SPACE_TIME_NEIGHBORS-Fills empty bins with the average value of
          space time neighbors.

          * TEMPORAL_TREND-Fills empty bins using an interpolated univariate
          spline algorithm.
          Note: Null values present in any of the summary fields will result in
          those features being excluded from analysis. If having the count of
          points in each bin is part of your analysis strategy, you may want to
          consider creating separate cubes, one for the count (without summary
          fields) and one for summary fields. If the set of null values is
          different for each summary field, you may also consider creating a
          separate cube for each summary field.
      aggregation_shape_type {String}:
          The shape of the polygon mesh into which the input feature point data
          will be aggregated.

          * FISHNET_GRID-The input features will be aggregated into a grid of
          square (fishnet) cells.

          * HEXAGON_GRID-The input features will be aggregated into a grid of
          hexagonal cells.

          * DEFINED_LOCATIONS-The input features will be aggregated into the
          locations provided.
      defined_polygon_locations {Feature Layer}:
          The polygon features into which the input point data will be
          aggregated. These can represent county boundaries, police beats, or
          sales territories, for example.
      location_id {Field}:
          The field containing the ID number for each unique location.

     OUTPUTS:
      output_cube (File):
          The output netCDF data cube that will be created to contain counts
          and summaries of the input feature point data."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateSpaceTimeCube_stpm(*gp_fixargs((in_features, output_cube, time_field, template_cube, time_step_interval, time_step_alignment, reference_time, distance_interval, summary_fields, aggregation_shape_type, defined_polygon_locations, location_id), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('CreateSpaceTimeCubeDefinedLocations_stpm', None)
def CreateSpaceTimeCubeDefinedLocations(in_features=None, output_cube=None, location_id=None, temporal_aggregation=None, time_field=None, time_step_interval=None, time_step_alignment=None, reference_time=None, variables=None, summary_fields=None, in_related_table=None, related_location_id=None):
    """CreateSpaceTimeCubeDefinedLocations_stpm(in_features, output_cube, location_id, temporal_aggregation, time_field, time_step_interval, {time_step_alignment}, {reference_time}, {variables;variables...}, {summary_fields;summary_fields...}, {in_related_table}, {related_location_id})

        Takes panel data or station data (defined locations where geography
        does not change but attributes are changing over time) and structures
        it into a netCDF data format by creating space-time bins. For all
        locations, the trend for variables or summary fields is evaluated.

     INPUTS:
      in_features (Feature Layer):
          The input point or polygon feature class to be converted into a space-
          time cube.
      location_id (Field):
          The field containing the ID number for each unique location.
      temporal_aggregation (Boolean):
          * APPLY_TEMPORAL_AGGREGATION-The space-time cube will temporally
          aggregate your features based on the time_step_interval you provide.
          For example, you have data that has been collected daily and want to
          create a cube with a weekly time_step_interval.

          * NO_TEMPORAL_AGGREGATION-The space-time cube will be created using
          the existing temporal structure of your in_features. For example, you
          have yearly data and want to create a cube with a yearly
          time_step_interval. This is the default.
      time_field (Field):
          The field containing the timestamp for each row in the dataset. This
          field must be of type Date.
      time_step_interval (Time unit):
          The number of seconds,  minutes, hours, days, weeks, or years that
          will represent a single time step. Examples of valid entries for this
          parameter are 1 Weeks, 13 Days, or 1 Months.If temporal_aggregation is
          checked off, you are not aggregating
          temporally, and this parameter should be set to the existing temporal
          structure of your data.If temporal_aggregation is checked on, you are
          aggregating temporally,
          and this parameter should be set to the time_step_interval you want to
          create. All features within the same time_step_interval will be
          aggregated.
      time_step_alignment {String}:
          Defines how the cube structure will occur based on a given
          time_step_interval.

          * END_TIME-Time steps align to the last time event and aggregate back
          in time.

          * START_TIME-Time steps align to the first time event and aggregate
          forward in time.

          * REFERENCE_TIME-Time steps align to a particular date/time that you
          specify. If all points in the input features have a timestamp larger
          than the reference time you provide (or it falls exactly on the start
          time of the input features), the time-step interval will begin with
          that reference time and aggregate forward in time (as occurs with a
          START_TIME alignment). If all points in the input features have a
          timestamp smaller than the reference time you provide (or it falls
          exactly on the end time of the input features), the time-step interval
          will end with that reference time and aggregate backward in time (as
          occurs with an END_TIME alignment). If the reference time you provide
          is in the middle of the time extent of your data, a time-step interval
          will be created ending with the reference time provided (as occurs
          with an END_TIME alignment); additional intervals will be created both
          before and after the reference time until the full time extent of your
          data is covered.
      reference_time {Date}:
          The date/time to used to align the time-step intervals. If you want
          to bin your data weekly from Monday to Sunday, for example, you could
          set a reference time of Sunday at midnight to ensure bins break
          between Sunday and Monday at midnight.
      variables {Value Table}:
          The numeric field containing attribute values that will be brought
          into the space-time cube.Available fill types are:

          * DROP_LOCATIONS-Locations with missing data for any of the variables
          will be dropped from the output space-time cube.

          * ZEROS-Fills empty bins with zeros.

          * SPATIAL_NEIGHBORS-Fills empty bins with the average value of spatial
          neighbors.

          * SPACE_TIME_NEIGHBORS-Fills empty bins with the average value of
          space time neighbors.

          * TEMPORAL_TREND-Fills empty bins using an interpolated univariate
          spline algorithm.
      summary_fields {Value Table}:
          The numeric field containing attribute values used to calculate the
          specified statistic when aggregating into a space-time cube. Multiple
          statistic and field combinations can be specified. Null values are
          excluded from all statistical calculations.Available statistic types
          are:

          * SUM-Adds the total value for the specified field within each bin.

          * MEAN-Calculates the average for the specified field within each bin.

          * MIN-Finds the smallest value for all records of the specified field
          within each bin.

          * MAX-Finds the largest value for all records of the specified field
          within each bin.

          * STD-Finds the standard deviation on values in the specified field
          within each bin.

          * MEDIAN-Finds the sorted middle value of all records of the specified
          field within each bin.
          Available fill types are:

          * ZEROS-Fills empty bins with zeros.

          * SPATIAL_NEIGHBORS-Fills empty bins with the average value of spatial
          neighbors

          * SPACE_TIME_NEIGHBORS-Fills empty bins with the average value of
          space time neighbors.

          * TEMPORAL_TREND-Fills empty bins using an interpolated univariate
          spline algorithm.
          Null values present in any of the summary fields will result in those
          features being excluded from analysis. If having the count of points
          in each bin is part of your analysis strategy, you may want to
          consider creating separate cubes, one for the count (without summary
          fields) and one for summary fields. If the set of null values is
          different for each summary field, you may also consider creating a
          separate cube for each summary field.
      in_related_table {Table View}:
          The table or table view to be related to the input features.
      related_location_id {Field}:
          The field in the Related Table that contains the location ID on which
          the relate will be based.

     OUTPUTS:
      output_cube (File):
          The output netCDF data cube that will be created."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateSpaceTimeCubeDefinedLocations_stpm(*gp_fixargs((in_features, output_cube, location_id, temporal_aggregation, time_field, time_step_interval, time_step_alignment, reference_time, variables, summary_fields, in_related_table, related_location_id), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('EmergingHotSpotAnalysis_stpm', None)
def EmergingHotSpotAnalysis(in_cube=None, analysis_variable=None, output_features=None, neighborhood_distance=None, neighborhood_time_step=None, polygon_mask=None, conceptualization_of_spatial_relationships=None, number_of_neighbors=None):
    """EmergingHotSpotAnalysis_stpm(in_cube, analysis_variable, output_features, {neighborhood_distance}, {neighborhood_time_step}, {polygon_mask}, {conceptualization_of_spatial_relationships}, {number_of_neighbors})

        Identifies trends in the clustering of point densities (counts) or
        values in a space-time cube created using either the Create Space Time
        Cube By Aggregating Points tool or the Create Space Time Cube From
        Defined Features tool. Categories include new, consecutive,
        intensifying, persistent, diminishing, sporadic, oscillating, and
        historical hot and cold spots.

     INPUTS:
      in_cube (File):
          The netCDF cube to be analyzed. This file must have an (.nc)
          extension and must have been created using the Create Space Time Cube
          By Aggregating Points tool or the Create Space Time Cube From Defined
          Features tool.
      analysis_variable (String):
          The numeric variable in the netCDF file you want to analyze.
      neighborhood_distance {Linear Unit}:
          The spatial extent of the analysis neighborhood. This value
          determines which features are analyzed together in order to assess
          local space-time clustering.
      neighborhood_time_step {Long}:
          The number of time-step intervals to include in the analysis
          neighborhood. This value determines which features are analyzed
          together in order to assess local space-time clustering.
      polygon_mask {Feature Layer}:
          A polygon feature layer with one or more polygons defining the
          analysis study area. You would use a polygon analysis mask to exclude
          a large lake from the analysis, for example. Bins defined in the
          in_cube that fall outside of the mask will not be included in the
          analysis.This parameter is only available for grid cubes.
      conceptualization_of_spatial_relationships {String}:
          Specifies how spatial relationships among bins are defined.

          * FIXED_DISTANCE-Each bin is analyzed within the context of
          neighboring bins. Neighboring bins inside the specified critical
          distance (neighborhood_distance) receive a weight of one and exert
          influence on computations for the target bin. Neighboring bins outside
          the critical distance receive a weight of zero and have no influence
          on a target bin's computations.

          * K_NEAREST_NEIGHBORS-The closest k bins are included in the analysis
          for the target bin; k is a specified numeric parameter.

          * CONTIGUITY_EDGES_ONLY-Only neighboring bins that share an edge will
          influence computations for the target polygon bin.

          * CONTIGUITY_EDGES_CORNERS-Bins that share an edge or share a node
          will influence computations for the target polygon bin.
      number_of_neighbors {Long}:
          An integer specifying either the minimum or the exact number of
          neighbors to include in calculations for the target bin. For
          K_NEAREST_NEIGHBORS, each bin will have exactly this specified number
          of neighbors. For FIXED_DISTANCE_BAND, each bin will have at least
          this many neighbors (the threshold distance will be temporarily
          extended to ensure this many neighbors if necessary). When one of the
          contiguity conceptualizations are selected, each bin will be assigned
          this minimum number of neighbors. For bins with fewer than this number
          of contiguous neighbors, additional neighbors will be based on feature
          centroid proximity.

     OUTPUTS:
      output_features (Feature Class):
          The output feature class results. This feature class will be a two-
          dimensional map representation of the hot and cold spot trends in your
          data. It will show, for example, any new or intensifying hot spots."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.EmergingHotSpotAnalysis_stpm(*gp_fixargs((in_cube, analysis_variable, output_features, neighborhood_distance, neighborhood_time_step, polygon_mask, conceptualization_of_spatial_relationships, number_of_neighbors), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('LocalOutlierAnalysis_stpm', None)
def LocalOutlierAnalysis(in_cube=None, analysis_variable=None, output_features=None, neighborhood_distance=None, neighborhood_time_step=None, number_of_permutations=None, polygon_mask=None, conceptualization_of_spatial_relationships=None, number_of_neighbors=None):
    """LocalOutlierAnalysis_stpm(in_cube, analysis_variable, output_features, {neighborhood_distance}, {neighborhood_time_step}, {number_of_permutations}, {polygon_mask}, {conceptualization_of_spatial_relationships}, {number_of_neighbors})

        Identifies statistically significant clusters and outliers in the
        context of both space and time. This tool is a space-time
        implementation of the Anselin Local Moran's I statistic.

     INPUTS:
      in_cube (File):
          The netCDF cube to be analyzed. This file must have an (.nc)
          extension and must have been created using either the Create Space
          Time Cube By Aggregating Points or Create Space Time Cube From Defined
          Features tool.
      analysis_variable (String):
          The numeric variable in the netCDF file you want to analyze.
      neighborhood_distance {Linear Unit}:
          The spatial extent of the analysis neighborhood. This value
          determines which features are analyzed together in order to assess
          local space-time clustering.
      neighborhood_time_step {Long}:
          The number of time-step intervals to include in the analysis
          neighborhood. This value determines which features are analyzed
          together in order to assess local space-time clustering.
      number_of_permutations {Long}:
          The number of random permutations for the calculation of pseudo
          p-values. The default number of permutations is 499. If you choose 0
          permutations, the standard p-value is calculated.

          * 0-Permutations are not used and a standard p-value is calculated.

          * 99-With 99 permutations, the smallest possible pseudo p-value is
          0.01 and all other pseudo p-values will be even multiples of this
          value.

          * 199-With 199 permutations, the smallest possible pseudo p-value is
          0.005 and all other pseudo p-values will be even multiples of this
          value.

          * 499-With 499 permutations, the smallest possible pseudo p-value is
          0.002 and all other pseudo p-values will be even multiples of this
          value.

          * 999-With 999 permutations, the smallest possible pseudo p-value is
          0.001 and all other pseudo p-values will be even multiples of this
          value.

          * 9999-With 9999 permutations, the smallest possible pseudo p-value is
          0.0001 and all other pseudo p-values will be even multiples of this
          value.
      polygon_mask {Feature Layer}:
          A polygon feature layer with one or more polygons defining the
          analysis study area. You would use a polygon analysis mask to exclude
          a large lake from the analysis, for example. Bins defined in the
          in_cube that fall outside of the mask will not be included in the
          analysis.This parameter is only available for grid cubes.
      conceptualization_of_spatial_relationships {String}:
          Specifies how spatial relationships among bins are defined.

          * FIXED_DISTANCE-Each bin is analyzed within the context of
          neighboring bins. Neighboring bins inside the specified critical
          distance (neighborhood_distance) receive a weight of one and exert
          influence on computations for the target bin. Neighboring bins outside
          the critical distance receive a weight of zero and have no influence
          on a target bin's computations.

          * K_NEAREST_NEIGHBORS-The closest k bins are included in the analysis
          for the target bin; k is a specified numeric parameter.

          * CONTIGUITY_EDGES_ONLY-Only neighboring bins that share an edge will
          influence computations for the target polygon bin.

          * CONTIGUITY_EDGES_CORNERS-Bins that share an edge or share a node
          will influence computations for the target polygon bin.
      number_of_neighbors {Long}:
          An integer specifying either the minimum or the exact number of
          neighbors to include in calculations for the target bin. For
          K_NEAREST_NEIGHBORS, each bin will have exactly this specified number
          of neighbors. For FIXED_DISTANCE, each bin will have at least this
          many neighbors (the neighborhood_distance will be temporarily extended
          to ensure this many neighbors if necessary). When one of the
          contiguity conceptualizations are selected, each bin will be assigned
          this minimum number of neighbors. For bins with fewer than this number
          of contiguous neighbors, additional neighbors will be based on feature
          centroid proximity.

     OUTPUTS:
      output_features (Feature Class):
          The output feature class containing locations that were considered
          statistically significant clusters or outliers."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.LocalOutlierAnalysis_stpm(*gp_fixargs((in_cube, analysis_variable, output_features, neighborhood_distance, neighborhood_time_step, number_of_permutations, polygon_mask, conceptualization_of_spatial_relationships, number_of_neighbors), True)))
        return retval
    except Exception as e:
        raise e


# Utilities toolset
@gptoolnotsupported
@gptooldoc('FillMissingValues_stpm', None)
def FillMissingValues(in_features=None, out_features=None, fields_to_fill=None, fill_method=None, conceptualization_of_spatial_relationships=None, distance_band=None, temporal_neighborhood=None, time_field=None, number_of_spatial_neighbors=None, location_id=None, related_table=None, related_location_id=None, weights_matrix_file=None, unique_id=None, null_value=None, out_table=None):
    """FillMissingValues_stpm(in_features, out_features, fields_to_fill;fields_to_fill..., fill_method, {conceptualization_of_spatial_relationships}, {distance_band}, {temporal_neighborhood}, {time_field}, {number_of_spatial_neighbors}, {location_id}, {related_table}, {related_location_id}, {weights_matrix_file}, {unique_id}, {null_value}, {out_table})

        Replaces missing values (null values) with estimated values based on
        spatial neighbors, space-time neighbors, or time-series values.

     INPUTS:
      in_features (Feature Layer):
          The feature class containing the null values to be filled.
      fields_to_fill (Field):
          The numeric fields containing missing data (null values).
      fill_method (String):
          The type of calculation that will be applied. TEMPORAL_TREND is only
          available if a location_id and time_field are specified.

          * AVERAGE      -Replaces null values with the average (mean) value of
          the feature's neighbors.

          * MINIMUM-Replaces null values with the minimum (smallest) value of
          the feature's neighbors.

          * MAXIMUM-Replaces null values with the maximum (largest) value of the
          feature's neighbors.

          * MEDIAN-Replaces null values with the median (sorted middle value) of
          the feature's neighbors.

          * TEMPORAL_TREND-Replaces null values based on the trend at that
          unique location.
      conceptualization_of_spatial_relationships {String}:
          Specifies how spatial relationships among features are defined.

          * FIXED_DISTANCE-Neighboring features within a specified critical
          distance (distance_band) of each feature are included in the
          calculations; everything outside the critical distance is excluded.

          * K_NEAREST_NEIGHBORS-The closest k features are included in the
          calculations; k is a specified numeric parameter.

          * CONTIGUITY_EDGES_ONLY-Only neighboring polygon features that share
          a boundary or overlap will influence computations for the target
          polygon feature.

          * CONTIGUITY_EDGES_CORNERS-Polygon features that share a boundary,
          share a node, or overlap will influence computations for the target
          polygon feature.

          * GET_SPATIAL_WEIGHTS_FROM_FILE-Spatial relationships are defined by a
          specified spatial weights file. The path to the spatial weights file
          is specified by the Weights_Matrix_File parameter.
      distance_band {Linear Unit}:
          Specifies a cutoff distance for the FIXED_DISTANCE option. Features
          outside the specified cutoff for a target feature are ignored in
          calculations for that feature. This parameter is not available for
          CONTIGUITY_EDGES_ONLY or CONTIGUITY_EDGES_CORNERS.
      temporal_neighborhood {Time unit}:
          Specifies an interval forward and backward in time to determine which
          features will be used in calculations for the target feature. Features
          that are not within this interval of the target feature are ignored in
          calculations for that feature.
      time_field {Field}:
          The field containing the time stamp for each record in the dataset.
          This field must be of type Date.This is required if a location_id has
          been provided.
      number_of_spatial_neighbors {Long}:
          The number of nearest neighbors to be included in calculations.If
          FIXED_DISTANCE, CONTIGUITY_EDGES_ONLY, or CONTIGUITY_EDGES_CORNERS
          is chosen, this number is the minimum number of neighbors to include
          in calculations.
      location_id {Field}:
          The field containing a unique ID number for each location.The
          location_id is used to match features from the in_features to rows
          in the related_table or to specify a unique location_id for
          determining temporal neighbors.
      related_table {Table View}:
          The table or table view containing the temporal data for each of the
          in_features.
      related_location_id {Field}:
          The field in the related_table that contains the location_id on
          which the relate will be based.
      weights_matrix_file {File}:
          The path to a file containing weights that define spatial, and
          potentially temporal, relationships among features.
      unique_id {Field}:
          An integer field containing a different value for every record in the
          in_features. This field can be used to join your results back to your
          original dataset.If you don't have a unique_id field, you can create
          one by adding an
          integer field to your feature class table and calculating the field
          values equal to the FID or OBJECTID field.
      null_value {Double}:
          The value that represents null (missing) values. If no value is
          specified, <Null> is assumed for geodatabase feature classes. For
          shapefile input, a numeric value of the null placeholder is required.

     OUTPUTS:
      out_features (Feature Class):
          The output that will include the filled (estimated) values.If a
          related_table is specified, the out_features contain the number
          of estimated values at each location, and the out_table contains the
          filled (estimated) values.
      out_table {Table}:
          The output table containing the filled (estimated) values.The output
          table is required if a related table is entered."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.FillMissingValues_stpm(*gp_fixargs((in_features, out_features, fields_to_fill, fill_method, conceptualization_of_spatial_relationships, distance_band, temporal_neighborhood, time_field, number_of_spatial_neighbors, location_id, related_table, related_location_id, weights_matrix_file, unique_id, null_value, out_table), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('VisualizeSpaceTimeCube2D_stpm', None)
def VisualizeSpaceTimeCube2D(in_cube=None, cube_variable=None, display_theme=None, output_features=None):
    """VisualizeSpaceTimeCube2D_stpm(in_cube, cube_variable, display_theme, output_features)

        Visualizes the variables stored in a netCDF cube and the results
        generated by the Space Time Pattern Mining tools. Output from this
        tool is a two-dimensional representation uniquely rendered based on
        the variable and theme chosen.

     INPUTS:
      in_cube (File):
          The netCDF cube that contains the variable to be displayed. This file
          must have an .nc extension and must have been created using either the
          Create Space Time Cube By Aggregating Points or Create Space Time Cube
          From Defined Locations tools.
      cube_variable (String):
          The numeric variable in the netCDF cube that you want to explore. The
          cube will always contain the COUNT variable.  Any Summary Fields or
          Variables will also be available if they were included in the cube
          creation process.
      display_theme (String):
          The characteristic of the Cube Variable that you want to display.
          Options will vary depending on how the cube was created and analyses
          run.If your cube was created by aggregating points, Locations with
          data
          and Trends will always be available. Number of estimated bins and
          Locations excluded from analysis will only be available for those
          Summary Fields that were included in the cube creation process.If your
          cube was created from defined locations, Trends will be
          available for those Summary Fields or Variables that were included in
          the cube creation process.Hot and cold spot trends and Emerging Hot
          Spot Analysis results will
          only be available after Emerging Hot Spot Analysis has been run on the
          selected Cube Variable. Percentage of local outliers, Local outlier in
          the most recent time period, Local Outlier Analysis results, and
          Locations without spatial neighbors are only available when you have
          run the Local Outlier Analysis tool.

          * LOCATIONS_WITH_DATA-Displays all locations that contain data for the
          chosen variable

          * TRENDS-The trend of values at each location using the Mann-Kendall
          statistic

          * HOT_AND_COLD_SPOT_TRENDS-The trend of z-scores at each location
          using the Mann-Kendall statistic

          * EMERGING_HOT_SPOT_ANALYSIS_RESULTS-Displays the results of Emerging
          Hot Spot Analysis for the Cube Variable chosen

          * LOCAL_OUTLIER_ANALYSIS_RESULTS-Displays the results of Local Outlier
          Analysis for the Cube Variable chosen

          * PERCENTAGE_OF_LOCAL_OUTLIERS-Displays the total percentage of
          outliers at each location

          * LOCAL_OUTLIER_IN_MOST_RECENT_TIME_PERIOD-Displays all outliers that
          are present in the most recent time period

          * LOCATIONS_WITHOUT_SPATIAL_NEIGHBORS-For the last analysis run,
          displays all locations that have no spatial neighbors and, as a
          result, are relying on temporal neighbors for analysis

          * NUMBER_OF_ESTIMATED_BINS-The number of bins that were estimated for
          each location

          * LOCATIONS_EXCLUDED_FROM_ANALYSIS-The locations that were excluded
          from analysis because they had empty bins that did not meet the
          criteria for estimation

     OUTPUTS:
      output_features (Feature Class):
          The output feature class results. This feature class will be a two-
          dimensional map representation of the display variable chosen."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.VisualizeSpaceTimeCube2D_stpm(*gp_fixargs((in_cube, cube_variable, display_theme, output_features), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('VisualizeSpaceTimeCube3D_stpm', None)
def VisualizeSpaceTimeCube3D(in_cube=None, cube_variable=None, display_theme=None, output_features=None):
    """VisualizeSpaceTimeCube3D_stpm(in_cube, cube_variable, display_theme, output_features)

        Visualizes the variables stored in a netCDF cube created with the
        Space Time Pattern Mining tools. Output from this tool is a three-
        dimensional representation uniquely rendered based on the variable and
        theme chosen.

     INPUTS:
      in_cube (File):
          The netCDF cube that contains the variable to be displayed. This file
          must have an .nc extension and must have been created using either the
          Create Space Time Cube By Aggregating Points or Create Space Time Cube
          From Defined Locations tool.
      cube_variable (String):
          The numeric variable in the netCDF cube that you want to explore. The
          cube will always contain the COUNT variable if aggregation was used in
          cube creation.  Any Summary Fields or Variables will also be available
          if they were included in the cube creation process.
      display_theme (String):
          The characteristic of the Cube Variable that you want to display.
          Options will vary depending on how the cube was created and analyses
          run.Value is the numeric value of the Cube Variable chosen. Estimated
          bins
          are only available for the Summary Fields that were included in the
          cube creation process. Hot and cold spot results will only be
          available for Cube Variables for which Emerging Hot Spot Analysis has
          been run. Cluster and outlier results will only be available for Cube
          Variables for which Local Outlier Analysis has been run. Temporal
          aggregation count will only be available for defined location cubes
          that have been aggregated temporally.

          * VALUE-The numeric value of the cube variable chosen

          * HOT_AND_COLD_SPOT_RESULTS-The statistical significance of each bin
          based on the space-time hot spot analysis run in Emerging Hot Spot
          Analysis

          * ESTIMATED_BINS-Displays the bins with estimated values

          * LOCAL_OUTLIER_RESULTS-The cluster or outlier type (COType) assigned
          to each bin from the analysis run in Local Outlier Analysis

          * TEMPORAL_AGGREGATION_COUNT-The count of records aggregated into each
          space-time bin.

     OUTPUTS:
      output_features (Feature Class):
          The output feature class results. This feature class will be a three-
          dimensional map representation of the display variable chosen that can
          be displayed in a 3D scene."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.VisualizeSpaceTimeCube3D_stpm(*gp_fixargs((in_cube, cube_variable, display_theme, output_features), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject, gptoolnotsupported